import logging
from spaceone.core.manager import BaseManager
from spaceone.notification.model.protocol_model import Protocol

_LOGGER = logging.getLogger(__name__)


class ProtocolManager(BaseManager):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.protocol_model: Protocol = self.locator.get_model('Protocol')

    def create_protocol(self, params):
        def _rollback(protocol_vo):
            _LOGGER.info(f'[create_protocol._rollback]'
                         f'Delete Protocol : {protocol_vo.name}'
                         f'({protocol_vo.protocol_id})')
            protocol_vo.delete()

        protocol_vo: Protocol = self.protocol_model.create(params)
        self.transaction.add_rollback(_rollback, protocol_vo)

        return protocol_vo

    def update_protocol(self, params):
        protocol_vo: Protocol = self.get_protocol(params['protocol_id'], params['domain_id'])
        return self.update_protocol_by_vo(params, protocol_vo)

    def update_protocol_by_vo(self, params, protocol_vo):
        def _rollback(old_data):
            _LOGGER.info(f'[update_protocol_by_vo._rollback] Revert Data : '
                         f'{old_data["protocol_id"]}')
            protocol_vo.update(old_data)

        self.transaction.add_rollback(_rollback, protocol_vo.to_dict())
        return protocol_vo.update(params)

    def delete_protocol(self, protocol_id, domain_id):
        self.delete_protocol_by_vo(self.get_protocol(protocol_id, domain_id))

    def enable_protocol(self, protocol_id, domain_id):
        def _rollback(old_data):
            _LOGGER.info(f'[enable_protocol._rollback] Revert Data : {old_data["name"]} ({old_data["protocol_id"]})')
            protocol_vo.update(old_data)

        protocol_vo: Protocol = self.get_protocol(protocol_id, domain_id)

        if protocol_vo.state != 'ENABLED':
            self.transaction.add_rollback(_rollback, protocol_vo.to_dict())
            protocol_vo.update({'state': 'ENABLED'})

        return protocol_vo

    def disable_protocol(self, protocol_id, domain_id):
        def _rollback(old_data):
            _LOGGER.info(f'[disable_protocol._rollback] Revert Data : {old_data["name"]} ({old_data["protocol_id"]})')
            protocol_vo.update(old_data)

        protocol_vo: Protocol = self.get_protocol(protocol_id, domain_id)

        if protocol_vo.state != 'DISABLED':
            self.transaction.add_rollback(_rollback, protocol_vo.to_dict())
            protocol_vo.update({'state': 'DISABLED'})

        return protocol_vo

    def get_protocol(self, protocol_id, domain_id, only=None):
        return self.protocol_model.get(protocol_id=protocol_id, domain_id=domain_id, only=only)

    def list_protocols(self, query={}):
        return self.protocol_model.query(**query)

    def stat_protocols(self, query):
        return self.protocol_model.stat(**query)

    @staticmethod
    def delete_protocol_by_vo(protocol_vo):
        return protocol_vo.delete()




from spaceone.core.error import *


class ERROR_JSON_SCHEMA(ERROR_BASE):
    _message = '{message}. (data={data})'


class ERROR_JSON_SCHEMA_PATTERN(ERROR_BASE):
    _message = 'Data is not matched {schema_title} pattern. (data={data})'



import logging
import functools

from spaceone.core.service import *
from spaceone.notification.error import *
from spaceone.notification.manager import ProtocolManager
from spaceone.notification.manager import QuotaManager
from spaceone.notification.model import Quota


_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class QuotaService(BaseService):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.quota_mgr: QuotaManager = self.locator.get_manager('QuotaManager')

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['protocol_id', 'limit', 'domain_id'])
    def create(self, params):
        """ Create Quota

        Args:
            params (dict): {
                'protocol_id': 'str',
                'limit': 'dict',
                'domain_id': 'str'
            }

        Returns:
            quota_vo (object)
        """
        protocol_mgr: ProtocolManager = self.locator.get_manager('ProtocolManager')
        params.update({
            'protocol': protocol_mgr.get_protocol(params['protocol_id'], params['domain_id']),
            'limit': self.limit_validation_check(params['limit'])
        })

        return self.quota_mgr.create_quota(params)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['quota_id', 'limit', 'domain_id'])
    def update(self, params):
        """ Update Quota

        Args:
            params (dict): {
                'quota_id': 'str',
                'limit': 'dict',
                'domain_id': 'str'
            }

        Returns:
            quota_vo (object)
        """
        quota_vo = self.quota_mgr.get_quota(params['quota_id'], params['domain_id'])

        params['limit'] = self.limit_validation_check(params['limit'])
        return self.quota_mgr.update_quota_by_vo(params, quota_vo)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['quota_id', 'domain_id'])
    def delete(self, params):
        """ Delete Quota

        Args:
            params (dict): {
                'quota_id': 'str',
                'domain_id': 'str'
            }

        Returns:
            None
        """
        quota_vo: Quota = self.quota_mgr.get_quota(params['quota_id'], params['domain_id'])
        return self.quota_mgr.delete_quota_by_vo(quota_vo)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['quota_id', 'domain_id'])
    def get(self, params):
        """ Get Quota

        Args:
            params (dict): {
                'quota_id': 'str',
                'domain_id': 'str'
            }

        Returns:
            quota_vo (object)
        """
        return self.quota_mgr.get_quota(params['quota_id'], params['domain_id'])

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['domain_id'])
    @append_query_filter(['quota_id', 'protocol_id',  'domain_id'])
    def list(self, params):
        """ List Quotas

        Args:
            params (dict): {
                'quota_id': 'str',
                'protocol_id': 'str',
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.Query)'
            }

        Returns:
            results (list): 'list of quota_vo'
            total_count (int)
        """
        query = params.get('query', {})
        return self.quota_mgr.list_quotas(query)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['query', 'domain_id'])
    @append_query_filter(['domain_id'])
    def stat(self, params):
        """
        Args:
            params (dict): {
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)'
            }

        Returns:
            values (list): 'list of statistics data'
            total_count (int)
        """

        query = params.get('query', {})
        return self.quota_mgr.stat_quotas(query)

    @staticmethod
    def limit_validation_check(limit):
        valid_limit = {}

        if 'day' in limit:
            if isinstance(limit['day'], int) or isinstance(limit['day'], float):
                if limit['day'] < -1:
                    raise ERROR_QUOTA_LIMIT_TYPE(limit=limit['day'])
            else:
                raise ERROR_QUOTA_LIMIT_TYPE(limit=limit['day'])

            valid_limit['day'] = int(limit['day'])

        if 'month' in limit:
            if isinstance(int(limit['month']), int) or isinstance(int(limit['month']), float):
                if limit['month'] < -1:
                    raise ERROR_QUOTA_LIMIT_TYPE(limit=limit['month'])
            else:
                raise ERROR_QUOTA_LIMIT_TYPE(limit=limit['month'])

            valid_limit['month'] = int(limit['month'])

        return valid_limit



import functools

from spaceone.api.notification.v1 import notification_pb2
from spaceone.core.pygrpc.message_type import *
from spaceone.core import utils
from spaceone.notification.model.notification_model import Notification

__all__ = ['NotificationInfo', 'NotificationsInfo']


def NotificationInfo(notification_vo: Notification, minimal=False):
    info = {
        'notification_id': notification_vo.notification_id,
        'topic': notification_vo.topic,
        'notification_type': notification_vo.notification_type,
        'notification_level': notification_vo.notification_level
    }

    if not minimal:

        if notification_vo.user_id:
            info.update({'user_id': notification_vo.user_id})

        info.update({
            'message': change_struct_type(notification_vo.message),
            'is_read': notification_vo.is_read,
            'created_at': utils.datetime_to_iso8601(notification_vo.created_at),
            'domain_id': notification_vo.domain_id
        })

    return notification_pb2.NotificationInfo(**info)


def NotificationsInfo(notification_vos, total_count, **kwargs):
    results = list(map(functools.partial(NotificationInfo, **kwargs), notification_vos))
    return notification_pb2.NotificationsInfo(results=results, total_count=total_count)



DEFAULT_INTERNAL_PROTOCOLS = [{
    "name": "SpaceONE User",
    "protocol_type": "INTERNAL",
    "resource_type": "identity.User",
    "capability": {
        'supported_schema': ['spaceone_user']
    },
    "plugin_info": {
        'metadata': {
            'data_type': 'PLAIN_TEXT'
        }
    },
    "tags": {
        "icon": "https://spaceone-custom-assets.s3.ap-northeast-2.amazonaws.com/console-assets/icons/notifications_member.svg"
    }
}]



import logging

from spaceone.core.manager import BaseManager
from spaceone.core.connector.space_connector import SpaceConnector

_LOGGER = logging.getLogger(__name__)

_GET_RESOURCE_METHODS = {
    'identity.Domain': {
        'dispatch_method': 'Domain.get',
        'key': 'domain_id'
    },
    'identity.Project': {
        'dispatch_method': 'Project.get',
        'key': 'project_id'
    },
    'identity.User': {
        'dispatch_method': 'User.get',
        'key': 'user_id'
    },
    'identity.ServiceAccount': {
        'dispatch_method': 'ServiceAccount.get',
        'key': 'service_account_id'
    },
}

class IdentityManager(BaseManager):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.identity_connector: SpaceConnector = self.locator.get_connector('SpaceConnector', service='identity')

    def get_resource(self, resource_id, resource_type, domain_id):
        get_method = _GET_RESOURCE_METHODS[resource_type]
        return self.identity_connector.dispatch(get_method['dispatch_method'], {get_method['key']: resource_id, 'domain_id': domain_id})

    def get_domain_info(self, domain_id):
        return self.identity_connector.dispatch('Domain.get', {'domain_id': domain_id})

    def get_all_users_in_domain(self, domain_id):
        query = {
            'state': 'ENABLED',
            'user_type': 'USER',
            'domain_id': domain_id
        }

        response = self.identity_connector.dispatch('User.list', query)
        return response.get('results', [])



from spaceone.api.notification.v1 import protocol_pb2, protocol_pb2_grpc
from spaceone.core.pygrpc import BaseAPI


class Protocol(BaseAPI, protocol_pb2_grpc.ProtocolServicer):

    pb2 = protocol_pb2
    pb2_grpc = protocol_pb2_grpc

    def create(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProtocolService', metadata) as protocol_svc:
            return self.locator.get_info('ProtocolInfo', protocol_svc.create(params))

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProtocolService', metadata) as protocol_svc:
            return self.locator.get_info('ProtocolInfo', protocol_svc.update(params))

    def update_plugin(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProtocolService', metadata) as protocol_svc:
            return self.locator.get_info('ProtocolInfo', protocol_svc.update_plugin(params))

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProtocolService', metadata) as protocol_svc:
            protocol_svc.delete(params)
            return self.locator.get_info('EmptyInfo')

    def enable(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProtocolService', metadata) as protocol_svc:
            return self.locator.get_info('ProtocolInfo', protocol_svc.enable(params))

    def disable(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProtocolService', metadata) as protocol_svc:
            return self.locator.get_info('ProtocolInfo', protocol_svc.disable(params))

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProtocolService', metadata) as protocol_svc:
            return self.locator.get_info('ProtocolInfo', protocol_svc.get(params))

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProtocolService', metadata) as protocol_svc:
            protocol_vos, total_count = protocol_svc.list(params)
            return self.locator.get_info('ProtocolsInfo',
                                         protocol_vos,
                                         total_count,
                                         minimal=self.get_minimal(params))

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProtocolService', metadata) as protocol_svc:
            return self.locator.get_info('StatisticsInfo', protocol_svc.stat(params))



from mongoengine import *
from spaceone.core.model.mongo_model import MongoModel
from spaceone.notification.model.schedule_model import Schedule


class UserChannel(MongoModel):
    user_channel_id = StringField(max_length=40, generate_id='user-ch', unique=True)
    user_id = StringField(max_length=255)
    protocol_id = StringField(max_length=40)
    name = StringField(max_length=255)
    state = StringField(max_length=20, default='ENABLED', choices=('ENABLED', 'DISABLED'))
    data = DictField()
    is_subscribe = BooleanField(default=False)
    subscriptions = ListField(StringField(max_length=255), default=[])
    is_scheduled = BooleanField(default=False)
    schedule = EmbeddedDocumentField(Schedule, default=None, null=True)
    tags = DictField()
    secret_id = StringField(max_length=255)
    domain_id = StringField(max_length=255)
    created_at = DateTimeField(auto_now_add=True)

    meta = {
        'updatable_fields': [
            'name',
            'state',
            'data',
            'is_subscribe',
            'subscriptions',
            'notification_level',
            'is_scheduled',
            'schedule',
            'tags'
        ],
        'minimal_fields': [
            'user_channel_id',
            'name',
            'state',
        ],
        'ordering': ['name'],
        'indexes': [
            # 'user_channel_id',
            'protocol_id',
            'state',
            'tags'
        ]
    }



from spaceone.notification.error.protocol import *
from spaceone.notification.error.channel import *
from spaceone.notification.error.plugin import *
from spaceone.notification.error.jsonschema import *
from spaceone.notification.error.quota import *



from mongoengine import *
from spaceone.core.model.mongo_model import MongoModel


class PluginInfo(EmbeddedDocument):
    plugin_id = StringField(max_length=255)
    version = StringField(max_length=255)
    options = DictField(default={})
    metadata = DictField(default={})
    secret_id = StringField(max_length=40)
    schema = StringField(max_length=255)
    upgrade_mode = StringField(max_length=255, choices=('AUTO', 'MANUAL'), default='AUTO')

    def to_dict(self):
        return dict(self.to_mongo())


class Protocol(MongoModel):
    protocol_id = StringField(max_length=40, generate_id='protocol', unique=True)
    name = StringField(max_length=255, unique_with=['domain_id'])
    state = StringField(max_length=20, default='ENABLED', choices=('ENABLED', 'DISABLED'))
    protocol_type = StringField(max_length=40, default='EXTERNAL', choices=('EXTERNAL', 'INTERNAL'))
    resource_type = StringField(max_length=40, null=True, default=None)
    capability = DictField()
    plugin_info = EmbeddedDocumentField(PluginInfo, default={})
    tags = DictField()
    domain_id = StringField(max_length=255)
    created_at = DateTimeField(auto_now_add=True)

    meta = {
        'updatable_fields': [
            'name',
            'state',
            'plugin_info',
            'tags'
        ],
        'minimal_fields': [
            'protocol_id',
            'name',
            'state',
        ],
        'ordering': ['name'],
        'indexes': [
            # 'protocol_id',
            'state',
            'tags'
        ]
    }



import functools

from spaceone.api.notification.v1 import protocol_pb2
from spaceone.core.pygrpc.message_type import *
from spaceone.core import utils
from spaceone.notification.model.protocol_model import Protocol

__all__ = ['ProtocolInfo', 'ProtocolsInfo']


def PluginInfo(plugin_info):
    if plugin_info:
        info = {
            'plugin_id': plugin_info.plugin_id,
            'version': plugin_info.version,
            'options': change_struct_type(plugin_info.options),
            'metadata': change_struct_type(plugin_info.metadata),
            'secret_id': plugin_info.secret_id,
            'upgrade_mode': plugin_info.upgrade_mode
        }
        return protocol_pb2.PluginInfo(**info)
    return None


def ProtocolInfo(protocol_vo: Protocol, minimal=False):
    info = {
        'protocol_id': protocol_vo.protocol_id,
        'name': protocol_vo.name,
        'state': protocol_vo.state
    }

    if not minimal:
        info.update({
            'capability': change_struct_type(protocol_vo.capability),
            'protocol_type': protocol_vo.protocol_type,
            'resource_type': protocol_vo.resource_type,
            'plugin_info': PluginInfo(protocol_vo.plugin_info),
            'created_at': utils.datetime_to_iso8601(protocol_vo.created_at),
            'tags': change_struct_type(protocol_vo.tags),
            'domain_id': protocol_vo.domain_id
        })

    return protocol_pb2.ProtocolInfo(**info)


def ProtocolsInfo(protocol_vos, total_count, **kwargs):
    results = list(map(functools.partial(ProtocolInfo, **kwargs), protocol_vos))

    return protocol_pb2.ProtocolsInfo(results=results, total_count=total_count)




DATABASE_AUTO_CREATE_INDEX = True
DATABASES = {
    'default': {
        # 'db': '',
        # 'host': '',
        # 'port': 0,
        # 'username': '',
        # 'password': '',
        # 'ssl': False,
        # 'ssl_ca_certs': ''
    }
}

CACHES = {
    'default': {},
    'local': {
        'backend': 'spaceone.core.cache.local_cache.LocalCache',
        'max_size': 128,
        'ttl': 300
    }
}

IDENTITY = {
    'token': {
        'token_timeout': 1800,
        'refresh_timeout': 3600,
        'refresh_ttl': 12,
        'refresh_once': True
    }
}

HANDLERS = {
    # 'authentication': [{
    #     'backend': 'spaceone.core.handler.authentication_handler.AuthenticationGRPCHandler',
    #     'uri': 'grpc://localhost:50051/v1/Domain/get_public_key'
    # }],
    # 'authorization': [{
    #     'backend': 'spaceone.core.handler.authorization_handler.AuthorizationGRPCHandler',
    #     'uri': 'grpc://localhost:50051/v1/Authorization/verify'
    # }],
    # 'mutation': [{
    #     'backend': 'spaceone.core.handler.mutation_handler.SpaceONEMutationHandler'
    # }],
    # 'event': []
}

CONNECTORS = {
    'PluginServiceConnector': {
    },
    'SpaceConnector': {
        'backend': 'spaceone.core.connector.space_connector.SpaceConnector',
        'endpoints': {
            'identity': 'grpc://identity:50051',
            'plugin': 'grpc://plugin:50051',
            'repository': 'grpc://repository:50051',
            'secret': 'grpc://secret:50051',
            'config': 'grpc://config:50051',
        }
    },
}

# Scheduler Settings
QUEUES = {}
SCHEDULERS = {}
WORKERS = {}
TOKEN = ""
TOKEN_INFO = {}

ENDPOINTS = [
    # {
    #     'service': 'notification',
    #     'name': 'Identity Service',
    #     'endpoint': 'grpc://<endpoint>>:<port>/v1'
    # },
]

INSTALLED_PROTOCOL_PLUGINS = [
    # {
    #     'name': 'Email',
    #     'plugin_info': {
    #         'plugin_id': 'plugin-email-noti-protocol',
    #         'version': '1.0',
    #         'options': {},
    #         'secret_data': {}
    #         'schema': 'email_smtp'
    #     },
    #     'tags':{
    #         'description': 'email'
    #     }
    # }
]

DEFAULT_QUOTA = {
    # ex: 'PROTOCOL_PLUGIN_ID': {'month': QUOTA, 'day': QUOTA},
    # 'plugin-slack-noti-protocol': {'month': -1, 'day': -1},
    # 'plugin-jira-noti-protocol': {'month': -1, 'day': -1},
    # 'plugin-mzc-sms-noti-protocol': {'month': 3000, 'day': 100},
    # 'plugin-mzc-voicecall-noti-protocol': {'month': 3000, 'day': 100},
    # 'plugin-telegram-noti-protocol': {'month': -1, 'day': -1},
    # 'plugin-email-noti-protocol': {'month': 30000, 'day': 1000},
}



import logging
from spaceone.core.manager import BaseManager
from spaceone.notification.model.notification_model import Notification
from spaceone.notification.error import *

_LOGGER = logging.getLogger(__name__)


class NotificationManager(BaseManager):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.notification_model: Notification = self.locator.get_model('Notification')

    def create_notification(self, params):
        def _rollback(notification_vo):
            _LOGGER.info(f'[create_protocol._rollback]'
                         f'Delete Notification : {notification_vo.name}'
                         f'({notification_vo.protocol_id})')
            notification_vo.delete()
        notification_vo: Notification = self.notification_model.create(params)
        self.transaction.add_rollback(_rollback, notification_vo)

        return notification_vo

    def set_read_notification(self, notifications, domain_id):
        def _rollback(notification_vos):
            _LOGGER.info(f'[set_read_notification._rollback]')
            notification_vos.update({'is_read': False})

        query = {'filter': [{'k': 'notification_id', 'v': notifications, 'o': 'in'},
                            {'k': 'domain_id', 'v': domain_id, 'o': 'eq'},
                            {'k': 'is_read', 'v': False, 'o': 'eq'}]}

        notification_vos, total_count = self.list_notifications(query)
        self.transaction.add_rollback(_rollback, notification_vos)
        notification_vos.update({'is_read': True})

    def get_notification(self, notification_id, domain_id, only=None):
        return self.notification_model.get(notification_id=notification_id, domain_id=domain_id, only=only)

    def list_notifications(self, query):
        return self.notification_model.query(**query)

    def stat_notifications(self, query):
        return self.notification_model.stat(**query)

    def filter_notifications(self, **conditions):
        return self.notification_model.filter(**conditions)

    @staticmethod
    def delete_notification_by_vos(notification_vos):
        notification_vos.delete()



from jsonschema import validate

from spaceone.core import utils
from spaceone.core.service import *
from spaceone.notification.error import *
from spaceone.notification.lib.schedule import *
from spaceone.notification.lib.schema import *
from spaceone.notification.manager import IdentityManager
from spaceone.notification.manager import ProtocolManager
from spaceone.notification.manager import ProjectChannelManager
from spaceone.notification.manager import SecretManager
from spaceone.notification.model import ProjectChannel


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class ProjectChannelService(BaseService):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.project_channel_mgr: ProjectChannelManager = self.locator.get_manager('ProjectChannelManager')
        self.identity_mgr: IdentityManager = self.locator.get_manager('IdentityManager')
        self.protocol_mgr: ProtocolManager = self.locator.get_manager('ProtocolManager')
        self.secret_mgr: SecretManager = self.locator.get_manager('SecretManager')

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['protocol_id', 'name', 'data', 'project_id', 'domain_id'])
    def create(self, params):

        """ Create Project Channel
        Args:
            params (dict): {
                'protocol_id': 'str',
                'name': 'str',
                'data': 'dict',
                'is_subscribe': 'bool',
                'subscriptions': 'list',
                'notification_level': 'str',
                'is_scheduled': 'bool',
                'schedule': 'dict',
                'project_id': 'str',
                'tags': 'dict',
                'domain_id': 'str'
            }

        Returns:
            project_channel_vo (object)
        """

        protocol_id = params['protocol_id']
        domain_id = params['domain_id']
        data = params['data']
        project_id = params['project_id']
        is_subscribe = params.get('is_subscribe', False)
        is_scheduled = params.get('is_scheduled', False)

        if not is_subscribe:
            params['subscriptions'] = []

        if is_scheduled:
            validate_schedule(params.get('schedule', {}))
        else:
            params['schedule'] = None

        self.identity_mgr.get_resource(project_id, 'identity.Project', domain_id)
        protocol_vo = self.protocol_mgr.get_protocol(protocol_id, domain_id)

        if protocol_vo.state == 'DISABLED':
            raise ERROR_PROTOCOL_DISABLED()

        metadata = protocol_vo.plugin_info.metadata
        schema = metadata.get('data', {}).get('schema')

        if schema:
            validate_json_schema(data, schema)

        if metadata['data_type'] == 'SECRET':
            new_secret_parameters = {
                'name': utils.generate_id('project-ch', 4),
                'secret_type': 'CREDENTIALS',
                'data': data,
                'project_id': project_id,
                'domain_id': domain_id
            }

            project_channel_secret = self.secret_mgr.create_secret(new_secret_parameters)

            params.update({
                'secret_id': project_channel_secret['secret_id'],
                'data': {}
            })

        # Create Project Channel
        return self.project_channel_mgr.create_project_channel(params)

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['project_channel_id', 'domain_id'])
    def update(self, params):
        """ Update project channel

        Args:
            params (dict): {
                'project_channel_id': 'str',
                'name': 'str',
                'data': 'dict',
                'notification_level': 'str',
                'tags': 'dict',
                'domain_id': 'str'
            }

        Returns:
            project_channel_vo (object)
        """
        project_channel_id = params['project_channel_id']
        domain_id = params['domain_id']

        project_channel_vo: ProjectChannel = self.project_channel_mgr.get_project_channel(project_channel_id, domain_id)

        if 'data' in params:
            protocol_vo = self.protocol_mgr.get_protocol(project_channel_vo.protocol_id, domain_id)
            metadata = protocol_vo.plugin_info.metadata
            schema = metadata.get('data', {}).get('schema')

            if schema:
                validate_json_schema(params['data'], schema)

            if project_channel_vo.secret_id:
                secret_params = {
                    'secret_id': project_channel_vo.secret_id,
                    'data': params['data'],
                    'domain_id': domain_id
                }

                self.secret_mgr.update_secret_data(secret_params)
                params['data'] = {}

        return self.project_channel_mgr.update_project_channel_by_vo(params, project_channel_vo)

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['project_channel_id', 'domain_id'])
    def set_schedule(self, params):
        """ Set schedule for Project Channel

        Args:
            params (dict): {
                'project_channel_id': 'str',
                'is_scheduled': bool,
                'schedule': dict,
                'domain_id': 'str'
            }

        Returns:
            project_channel_vo (object)
        """
        project_channel_vo = self.project_channel_mgr.get_project_channel(params['project_channel_id'],
                                                                          params['domain_id'])

        is_scheduled = params.get('is_scheduled', False)

        if is_scheduled:
            validate_schedule(params.get('schedule', {}))
        else:
            params.update({
                'is_scheduled': False,
                'schedule': None
            })

        return self.project_channel_mgr.update_project_channel_by_vo(params, project_channel_vo)

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['project_channel_id', 'domain_id'])
    def set_subscription(self, params):
        """ Set subscriptions for Project Channel

        Args:
            params (dict): {
                'project_channel_id': 'str',
                'is_subscribe': bool,
                'subscriptions': list,
                'domain_id': 'str'
            }

        Returns:
            project_channel_vo (object)
        """
        if not params.get('is_subscribe', False):
            params.update({
                'is_subscribe': False,
                'subscriptions': []
            })

        return self.project_channel_mgr.update_project_channel(params)

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['project_channel_id', 'domain_id'])
    def delete(self, params):
        """ Delete project channel

        Args:
            params (dict): {
                'project_channel_id': 'str',
                'domain_id': 'str'
            }

        Returns:
            None
        """
        project_channel_id = params['project_channel_id']
        domain_id = params['domain_id']

        project_channel_vo = self.project_channel_mgr.get_project_channel(project_channel_id, domain_id)

        if secret_id := project_channel_vo.secret_id:
            self.secret_mgr.delete_secret({'secret_id': secret_id, 'domain_id': domain_id})

        self.project_channel_mgr.delete_project_channel_by_vo(project_channel_vo)

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['project_channel_id', 'domain_id'])
    def enable(self, params):
        """ Enable project channel

        Args:
            params (dict): {
                'project_channel_id': 'str',
                'domain_id': 'str'
            }

        Returns:
            project_channel_vo (object)
        """

        return self.project_channel_mgr.enable_project_channel(params['project_channel_id'], params['domain_id'])

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['project_channel_id', 'domain_id'])
    def disable(self, params):
        """ Disable project channel

        Args:
            params (dict): {
                'project_channel_id': 'str',
                'domain_id': 'str'
            }

        Returns:
            project_channel_vo (object)
        """

        return self.project_channel_mgr.disable_project_channel(params['project_channel_id'], params['domain_id'])

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['project_channel_id', 'domain_id'])
    def get(self, params):
        """ Get Project Channel

        Args:
            params (dict): {
                'project_channel_id': 'str',
                'only': 'list'
            }

        Returns:
            project_channel_vo (object)
        """

        return self.project_channel_mgr.get_project_channel(params['project_channel_id'],
                                                            params['domain_id'],
                                                            params.get('only'))

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['domain_id'])
    @append_query_filter(['project_channel_id', 'name', 'state', 'secret_id', 'is_subscribe', 'is_scheduled',
                          'notification_level', 'protocol_id', 'project_id', 'domain_id', 'user_projects'])
    @append_keyword_filter(['project_channel_id', 'name'])
    def list(self, params):
        """ List Project Channels

        Args:
            params (dict): {
                'project_channel_id': 'str',
                'name': 'str',
                'state': 'str',
                'secret_id': 'str',
                'is_subscribe': 'bool',
                'is_scheduled': 'bool',
                'notification_level': 'str',
                'protocol_id': 'str',
                'project_id': 'str',
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.Query)',
                'user_projects': 'list', // from meta
            }

        Returns:
            results (list): 'list of project_channel_vo'
            total_count (int)
        """
        query = params.get('query', {})
        return self.project_channel_mgr.list_project_channels(query)

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['query', 'domain_id'])
    @append_query_filter(['domain_id', 'user_projects'])
    @append_keyword_filter(['project_channel_id', 'name'])
    def stat(self, params):
        """
        Args:
            params (dict): {
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)',
                'user_projects': 'list', // from meta
            }

        Returns:
            values (list): 'list of statistics data'
            total_count (int)
        """
        query = params.get('query', {})
        return self.project_channel_mgr.stat_project_channels(query)



from spaceone.notification.info.common_info import *
from spaceone.notification.info.protocol_info import *
from spaceone.notification.info.project_channel_info import *
from spaceone.notification.info.user_channel_info import *
from spaceone.notification.info.notification_info import *
from spaceone.notification.info.quota_info import *
from spaceone.notification.info.notification_usage_info import *



import functools

from spaceone.api.notification.v1 import user_channel_pb2
from spaceone.core.pygrpc.message_type import *
from spaceone.core import utils
from spaceone.notification.model.user_channel_model import UserChannel

__all__ = ['UserChannelInfo', 'UserChannelsInfo']


def ScheduleInfo(schedule_info):

    info = {
        'day_of_week': schedule_info.day_of_week,
        'start_hour': schedule_info.start_hour,
        'end_hour': schedule_info.end_hour
    }

    return user_channel_pb2.UserChannelSchedule(**info)


def UserChannelInfo(user_channel_vo: UserChannel, minimal=False):
    info = {
        'user_channel_id': user_channel_vo.user_channel_id,
        'name': user_channel_vo.name,
        'state': user_channel_vo.state
    }

    if not minimal:
        info.update({
            'data': change_struct_type(user_channel_vo.data),
            'secret_id': user_channel_vo.secret_id,
            'is_subscribe': user_channel_vo.is_subscribe,
            'subscriptions': change_list_value_type(user_channel_vo.subscriptions),
            'protocol_id': user_channel_vo.protocol_id,
            'user_id': user_channel_vo.user_id,
            'is_scheduled': user_channel_vo.is_scheduled,
            'schedule': ScheduleInfo(user_channel_vo.schedule) if user_channel_vo.schedule else user_channel_vo.schedule,
            'created_at': utils.datetime_to_iso8601(user_channel_vo.created_at),
            'tags': change_struct_type(user_channel_vo.tags),
            'domain_id': user_channel_vo.domain_id
        })

    return user_channel_pb2.UserChannelInfo(**info)


def UserChannelsInfo(user_channel_vos, total_count, **kwargs):
    results = list(map(functools.partial(UserChannelInfo, **kwargs), user_channel_vos))
    return user_channel_pb2.UserChannelsInfo(results=results, total_count=total_count)


from spaceone.notification.model.protocol_model import Protocol
from spaceone.notification.model.notification_model import Notification
from spaceone.notification.model.project_channel_model import ProjectChannel
from spaceone.notification.model.user_channel_model import UserChannel
from spaceone.notification.model.schedule_model import Schedule
from spaceone.notification.model.quota_model import Quota
from spaceone.notification.model.notification_usage_model import NotificationUsage



from mongoengine import *
from spaceone.core.model.mongo_model import MongoModel


class Schedule(EmbeddedDocument):
    day_of_week = ListField(StringField(choices=['MON', 'TUE', 'WED', 'THU', 'FRI', 'SAT', 'SUN'], required=True))
    start_hour = IntField(required=True, default=0)
    end_hour = IntField(required=True)



"""
Deprecated:
  Not used. Integrated with SpaceConnector.
"""

import logging

from google.protobuf.json_format import MessageToDict

from spaceone.core.connector import BaseConnector
from spaceone.core import pygrpc
from spaceone.core.utils import parse_endpoint
from spaceone.core.error import *

__all__ = ['SecretConnector']

_LOGGER = logging.getLogger(__name__)


class SecretConnector(BaseConnector):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._check_config()
        self._init_client()

    def _init_client(self):
        for version, uri in self.config['endpoint'].items():
            e = parse_endpoint(uri)
            self.client = pygrpc.client(endpoint=f'{e.get("hostname")}:{e.get("port")}', version=version)

    def _check_config(self):
        if 'endpoint' not in self.config:
            raise ERROR_CONNECTOR_CONFIGURATION(connector=self.__class__.__name__)

        if len(self.config['endpoint']) > 1:
            raise ERROR_CONNECTOR_CONFIGURATION(connector=self.__class__.__name__)

    def create_secret(self, params):
        response = self.client.Secret.create(params, metadata=self.transaction.get_connection_meta())
        return self._change_message(response)

    def update_secret(self, params):
        response = self.client.Secret.update(params, metadata=self.transaction.get_connection_meta())
        return self._change_message(response)

    def update_secret_data(self, params):
        response = self.client.Secret.update_data(params, metadata=self.transaction.get_connection_meta())
        return self._change_message(response)

    def delete_secret(self, params):
        self.client.Secret.delete(params, metadata=self.transaction.get_connection_meta())

    def list_secrets(self, query, domain_id):
        response = self.client.Secret.list({
            'query': query,
            'domain_id': domain_id
        }, metadata=self.transaction.get_connection_meta())

        return self._change_message(response)

    def get_secret_data(self, secret_id, domain_id):
        response = self.client.Secret.get_data({
            'secret_id': secret_id,
            'domain_id': domain_id
        }, metadata=self.transaction.get_connection_meta())

        return self._change_message(response)

    @staticmethod
    def _change_message(message):
        return MessageToDict(message, preserving_proto_field_name=True)



import functools

from spaceone.api.notification.v1 import project_channel_pb2
from spaceone.core.pygrpc.message_type import *
from spaceone.core import utils
from spaceone.notification.model.project_channel_model import ProjectChannel

__all__ = ['ProjectChannelInfo', 'ProjectChannelsInfo']


def ScheduleInfo(schedule_info):
    info = {
        'day_of_week': schedule_info.day_of_week,
        'start_hour': schedule_info.start_hour,
        'end_hour': schedule_info.end_hour
    }
    return project_channel_pb2.ProjectChannelSchedule(**info)


def ProjectChannelInfo(project_channel_vo: ProjectChannel, minimal=False):
    info = {
        'project_channel_id': project_channel_vo.project_channel_id,
        'name': project_channel_vo.name,
        'state': project_channel_vo.state
    }

    if not minimal:
        info.update({
            'data': change_struct_type(project_channel_vo.data),
            'secret_id': project_channel_vo.secret_id,
            'is_subscribe': project_channel_vo.is_subscribe,
            'subscriptions': change_list_value_type(project_channel_vo.subscriptions),
            'notification_level': project_channel_vo.notification_level,
            'protocol_id': project_channel_vo.protocol_id,
            'project_id': project_channel_vo.project_id,
            'is_scheduled': project_channel_vo.is_scheduled,
            'schedule': ScheduleInfo(project_channel_vo.schedule) if project_channel_vo.schedule else project_channel_vo.schedule,
            'created_at': utils.datetime_to_iso8601(project_channel_vo.created_at),
            'tags': change_struct_type(project_channel_vo.tags),
            'domain_id': project_channel_vo.domain_id
        })

    return project_channel_pb2.ProjectChannelInfo(**info)


def ProjectChannelsInfo(project_channel_vos, total_count, **kwargs):
    results = list(map(functools.partial(ProjectChannelInfo, **kwargs), project_channel_vos))
    return project_channel_pb2.ProjectChannelsInfo(results=results, total_count=total_count)


from mongoengine import *
from spaceone.core.model.mongo_model import MongoModel


class NotificationUsage(MongoModel):
    protocol_id = StringField(max_length=40)
    usage_date = StringField(max_length=255)
    usage_month = StringField(max_length=255)
    count = IntField(default=0)
    fail_count = IntField(default=0)
    domain_id = StringField(max_length=255)

    meta = {
        'updatable_fields': [
            'count',
            'fail_count'
        ],
        'minimal_fields': [
            'protocol_id',
            'usage_date',
            'usage_month',
            'count',
            'fail_count',
            'domain_id',
        ],
        'ordering': ['protocol_id', 'usage_month', 'usage_date'],
        'indexes': [
            'protocol_id',
            'usage_date',
            'usage_month',
            'domain_id'
        ]
    }



from spaceone.core.error import *


class ERROR_INVALID_PLUGIN_VERSION(ERROR_INVALID_ARGUMENT):
    _message = 'Plugin version is invalid. (plugin_id = {plugin_id}, version = {version})'


class ERROR_NOT_FOUND_PLUGIN_VERSION(ERROR_BASE):
    _message = 'Not Found Plugin version (plugin_id= {plugin_id})'


class ERROR_SUPPORTED_SECRETS_NOT_EXISTS(ERROR_INVALID_ARGUMENT):
    _message = 'There are no secrets that support plugins. (plugin_id = {plugin_id}, provider = {provider})'


class ERROR_RESOURCE_SECRETS_NOT_EXISTS(ERROR_INVALID_ARGUMENT):
    _message = 'There are no secrets in the resources. (resource_id = {resource_id})'


class ERROR_WRONG_PLUGIN_SETTINGS(ERROR_BASE):
    _message = "The plugin settings is incorrect. (key = {key})"


class ERROR_NOT_ALLOWED_UPDATE_PROTOCOL_TYPE(ERROR_BASE):
    _message = 'default porotocol type is not allowed to update. (protocol_id = {protocol_id})'


class EROR_DELETE_PROJECT_EXITED_CHANNEL(ERROR_BASE):
    _message = '{protocol_id} was used in Project or User Channel'


import logging
from spaceone.core.manager import BaseManager
from spaceone.notification.model.quota_model import Quota

_LOGGER = logging.getLogger(__name__)


class QuotaManager(BaseManager):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.quota_model: Quota = self.locator.get_model('Quota')

    def create_quota(self, params):
        def _rollback(quota_vo):
            _LOGGER.info(f'[create_quota._rollback]'
                         f'Delete Quota : {quota_vo.quota_id}'
                         f'({quota_vo.quota_id})')
            quota_vo.delete()

        quota_vo: Quota = self.quota_model.create(params)
        self.transaction.add_rollback(_rollback, quota_vo)

        return quota_vo

    def update_quota(self, params):
        quota_vo: Quota = self.get_quota(params['quota_id'], params['domain_id'])
        return self.update_quota_by_vo(params, quota_vo)

    def update_quota_by_vo(self, params, quota_vo):
        def _rollback(old_data):
            _LOGGER.info(f'[update_quota_by_vo._rollback] Revert Data : '
                         f'{old_data["quota_id"]}')
            quota_vo.update(old_data)

        self.transaction.add_rollback(_rollback, quota_vo.to_dict())
        return quota_vo.update(params)

    def delete_quota(self, quota_id, domain_id):
        self.delete_quota_by_vo(self.get_quota(quota_id, domain_id))

    def get_quota(self, quota_id, domain_id, only=None):
        return self.quota_model.get(quota_id=quota_id, domain_id=domain_id, only=only)

    def list_quotas(self, query={}):
        return self.quota_model.query(**query)

    def stat_quotas(self, query):
        return self.quota_model.stat(**query)

    @staticmethod
    def delete_quota_by_vo(quota_vo):
        return quota_vo.delete()




PROTO = {
    'spaceone.notification.interface.grpc.v1.protocol': ['Protocol'],
    'spaceone.notification.interface.grpc.v1.quota': ['Quota'],
    'spaceone.notification.interface.grpc.v1.project_channel': ['ProjectChannel'],
    'spaceone.notification.interface.grpc.v1.user_channel': ['UserChannel'],
    'spaceone.notification.interface.grpc.v1.notification': ['Notification'],
    'spaceone.notification.interface.grpc.v1.notification_usage': ['NotificationUsage']
}



import functools

from spaceone.api.notification.v1 import notification_usage_pb2
from spaceone.notification.model.notification_usage_model import NotificationUsage

__all__ = ['NotificationUsageInfo', 'NotificationUsagesInfo']


def NotificationUsageInfo(noti_usage_vo: NotificationUsage, minimal=False):
    info = {
        'protocol_id': noti_usage_vo.protocol_id,
        'usage_month': noti_usage_vo.usage_month,
        'usage_date': noti_usage_vo.usage_date,
        'count': noti_usage_vo.count,
        'fail_count': noti_usage_vo.fail_count,
        'domain_id': noti_usage_vo.domain_id
    }

    return notification_usage_pb2.NotificationUsageInfo(**info)


def NotificationUsagesInfo(noti_usage_vos, total_count, **kwargs):
    results = list(map(functools.partial(NotificationUsageInfo, **kwargs), noti_usage_vos))
    return notification_usage_pb2.NotificationUsagesInfo(results=results, total_count=total_count)



from spaceone.core.error import *


class ERROR_QUOTA_LIMIT_TYPE(ERROR_BASE):
    _message = '{limit} is invalid type.'


class ERROR_QUOTA_IS_EXCEEDED(ERROR_BASE):
    _message = 'Dispatch a notification is not possible because the quota is exceeded. ' \
               '(Protocol={protocol_id}, {limit})'



import logging

from google.protobuf.json_format import MessageToDict

from spaceone.core.connector import BaseConnector
from spaceone.core import pygrpc
from spaceone.core.utils import parse_endpoint

__all__ = ['NotificationPluginConnector']
_LOGGER = logging.getLogger(__name__)


class NotificationPluginConnector(BaseConnector):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.client = None

    def initialize(self, endpoint):
        static_endpoint = self.config.get('endpoint')

        if static_endpoint:
            endpoint = static_endpoint

        e = parse_endpoint(endpoint)
        self.client = pygrpc.client(endpoint=f'{e.get("hostname")}:{e.get("port")}', version='plugin')

    def init(self, options):
        response = self.client.Protocol.init({
            'options': options,
        }, metadata=self.transaction.get_connection_meta())

        return self._change_message(response)

    def verify(self, options, secret_data):
        params = {
            'options': options,
            'secret_data': secret_data
        }

        self.client.Protocol.verify(params, metadata=self.transaction.get_connection_meta())

    def dispatch_notification(self, secret_data, channel_data, notification_type, message, options={}):
        params = {
            'secret_data': secret_data,
            'channel_data': channel_data,
            'notification_type': notification_type,
            'message': message,
            'options': options,
        }

        response = self.client.Notification.dispatch(params, metadata=self.transaction.get_connection_meta())
        return self._change_message(response)

    @staticmethod
    def _change_message(message):
        return MessageToDict(message, preserving_proto_field_name=True)



from spaceone.notification.manager.protocol_manager import ProtocolManager
from spaceone.notification.manager.project_channel_manager import ProjectChannelManager
from spaceone.notification.manager.user_channel_manager import UserChannelManager
from spaceone.notification.manager.repository_manager import RepositoryManager
from spaceone.notification.manager.identity_manager import IdentityManager
from spaceone.notification.manager.secret_manager import SecretManager
from spaceone.notification.manager.plugin_manager import PluginManager
from spaceone.notification.manager.notification_manager import NotificationManager
from spaceone.notification.manager.quota_manager import QuotaManager
from spaceone.notification.manager.notification_usage_manager import NotificationUsageManager



import logging

from spaceone.core import config
from spaceone.core.locator import Locator
from spaceone.core.scheduler import IntervalScheduler

_LOGGER = logging.getLogger(__name__)


class DeleteOldNotificationScheduler(IntervalScheduler):

    def __init__(self, queue, interval):
        super().__init__(queue, interval)
        self.locator = Locator()
        self._init_config()
        self._create_metadata()

    def _init_config(self):
        self._token = config.get_global('TOKEN')

    def _create_metadata(self):
        self._metadata = {
            'token': self._token,
            'service': 'notification',
            'resource': 'Notification',
            'verb': 'delete_old_notifications'
        }

    def create_task(self):
        task = {
            'name': 'delete_old_notification_scheduler',
            'version': 'v1',
            'executionEngine': 'BaseWorker',
            'stages': [{
                'locator': 'SERVICE',
                'name': 'NotificationService',
                'metadata': self._metadata,
                'method': 'delete_old_notifications',
                'params': {
                    'params': {}
                }
            }]
        }

        return [task]



import logging
import datetime

from spaceone.core import queue, utils
from spaceone.core.service import *
from spaceone.notification.lib.schedule import *
from spaceone.notification.manager import IdentityManager
from spaceone.notification.manager import NotificationManager
from spaceone.notification.manager import ProjectChannelManager
from spaceone.notification.manager import UserChannelManager
from spaceone.notification.manager import ProtocolManager
from spaceone.notification.manager import SecretManager
from spaceone.notification.manager import PluginManager
from spaceone.notification.manager import QuotaManager
from spaceone.notification.manager import NotificationUsageManager
from spaceone.notification.conf.global_conf import *

_LOGGER = logging.getLogger(__name__)
OLD_NOTIFICATION_DAYS = 60


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class NotificationService(BaseService):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.notification_mgr: NotificationManager = self.locator.get_manager('NotificationManager')

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['resource_type', 'resource_id', 'topic', 'message', 'domain_id'])
    def create(self, params):
        """ Create Notification

        Args:
            params (dict): {
                'resource_type': 'str' -> identity.Project | identity.User,
                'resource_id': 'str',
                'topic': 'str',
                'message': 'dict',
                'notification_type': 'str' -> INFO(default) | ERROR | SUCCESS | WARNING,
                'notification_level': 'str' -> ALL(default) | LV1 | LV2 | LV3 | LV4 | LV5,
                'domain_id': 'str'
            }

        Returns:
            notification_vo (object)
        """
        identity_mgr: IdentityManager = self.locator.get_manager('IdentityManager')

        domain_id = params['domain_id']
        resource_type = params['resource_type']
        resource_id = params['resource_id']
        message = params['message']

        domain_info = identity_mgr.get_domain_info(domain_id)
        message['domain_name'] = self.get_domain_name(domain_info)

        identity_mgr.get_resource(resource_id, resource_type, domain_id)

        if resource_type == 'identity.Domain':
            self.dispatch_domain(params)

        elif resource_type == 'identity.Project':
            self.dispatch_project_channel(params)

        elif resource_type == 'identity.User':
            self.dispatch_user_channel(params)

    def dispatch_domain(self, params):
        _LOGGER.debug(f'[Dispatch Domain] Domain ID: {params["resource_id"]}')

        identity_mgr: IdentityManager = self.locator.get_manager('IdentityManager')
        users = identity_mgr.get_all_users_in_domain(params['resource_id'])

        for user in users:
            user_channel_params = {
                'notification_type': params.get('notification_type', 'INFO'),
                'notification_level': params.get('notification_level', 'ALL'),
                'topic': params['topic'],
                'message': params['message'],
                'resource_type': 'identity.User',
                'resource_id': user['user_id'],
                'domain_id': params['resource_id']
            }

            self.dispatch_user_channel(user_channel_params)

    def dispatch_project_channel(self, params):
        _LOGGER.debug(f'[Dispatch Project Channel] Project ID: {params["resource_id"]}')
        protocol_mgr: ProtocolManager = self.locator.get_manager('ProtocolManager')
        project_ch_mgr: ProjectChannelManager = self.locator.get_manager('ProjectChannelManager')

        domain_id = params['domain_id']
        topic = params['topic']
        resource_id = params['resource_id']

        notification_type = params.get('notification_type', 'INFO')
        notification_level = params.get('notification_level', 'ALL')
        message = params['message']

        prj_ch_vos, prj_ch_total_count = project_ch_mgr.list_project_channels(
            {'filter': [{'k': 'project_id', 'v': resource_id, 'o': 'eq'},
                        {'k': 'domain_id', 'v': domain_id, 'o': 'eq'}]})

        for prj_ch_vo in prj_ch_vos:
            if prj_ch_vo.state == 'ENABLED':
                protocol_vo = protocol_mgr.get_protocol(prj_ch_vo.protocol_id, domain_id)

                dispatch_subscribe = self.check_subscribe_for_dispatch(prj_ch_vo.is_subscribe, prj_ch_vo.subscriptions,
                                                                       topic)
                dispatch_schedule = self.check_schedule_for_dispatch(prj_ch_vo.is_scheduled, prj_ch_vo.schedule)
                dispatch_notification_level = self.check_notification_level_for_dispatch(notification_level,
                                                                                         prj_ch_vo.notification_level)

                _LOGGER.debug(f'[Notification] subscribe: {dispatch_subscribe} | schedule: {dispatch_schedule} '
                              f'| notification_level: {dispatch_notification_level}')

                if dispatch_subscribe and dispatch_schedule and dispatch_notification_level:
                    _LOGGER.info(f'[Notification] Dispatch Notification to project: {resource_id}')

                    if protocol_vo.protocol_type == 'INTERNAL':
                        internal_project_channel_data = prj_ch_vo.data
                        for user_id in internal_project_channel_data.get('users', []):
                            params.update({
                                'resource_type': 'identity.User',
                                'resource_id': user_id
                            })
                            _LOGGER.debug(f'[Forward to User Channel] User ID: {user_id}')
                            self.dispatch_user_channel(params)
                    elif protocol_vo.protocol_type == 'EXTERNAL':
                        _LOGGER.info(f'[Notification] Dispatch Notification to project: {resource_id}')
                        channel_data = self.get_channel_data(prj_ch_vo, protocol_vo, domain_id)
                        secret_data = self.get_secret_data(protocol_vo, domain_id)

                        self.push_queue(protocol_vo.protocol_id, channel_data, secret_data, notification_type, message, domain_id)
                else:
                    _LOGGER.info(f'[Notification] Skip Notification to project: {resource_id}')
            else:
                _LOGGER.info(f'[Notification] Project Channel is disabled: {prj_ch_vo.project_channel_id}')

    def dispatch_user_channel(self, params):
        _LOGGER.debug(f'[Dispatch User Channel] User ID: {params["resource_id"]}')

        protocol_mgr: ProtocolManager = self.locator.get_manager('ProtocolManager')
        user_ch_mgr: UserChannelManager = self.locator.get_manager('UserChannelManager')

        domain_id = params['domain_id']
        topic = params['topic']
        resource_id = params['resource_id']

        notification_type = params.get('notification_type', 'INFO')
        message = params['message']

        user_ch_vos, user_ch_total_count = user_ch_mgr.list_user_channels(
            {'filter': [{'k': 'user_id', 'v': resource_id, 'o': 'eq'},
                        {'k': 'domain_id', 'v': domain_id, 'o': 'eq'}]})

        for user_ch_vo in user_ch_vos:
            if user_ch_vo.state == 'ENABLED':
                protocol_vo = protocol_mgr.get_protocol(user_ch_vo.protocol_id, domain_id)

                dispatch_subscribe = self.check_subscribe_for_dispatch(user_ch_vo.is_subscribe,
                                                                       user_ch_vo.subscriptions, topic)
                dispatch_schedule = self.check_schedule_for_dispatch(user_ch_vo.is_scheduled, user_ch_vo.schedule)

                _LOGGER.debug(f'[Notification] subscribe: {dispatch_subscribe} | schedule: {dispatch_schedule}')

                if dispatch_subscribe and dispatch_schedule:
                    _LOGGER.info(f'[Notification] Dispatch Notification to user: {resource_id}')
                    channel_data = self.get_channel_data(user_ch_vo, protocol_vo, domain_id)
                    secret_data = self.get_secret_data(protocol_vo, domain_id)

                    self.push_queue(protocol_vo.protocol_id, channel_data, secret_data, notification_type, message, domain_id)
                else:
                    _LOGGER.info(f'[Notification] Skip Notification to user: {resource_id}')
            else:
                _LOGGER.info(f'[Notification] User Channel is disabled: {user_ch_vo.user_channel_id}')

        params.update({'user_id': resource_id})
        self.notification_mgr.create_notification(params)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['protocol_id', 'data', 'message', 'domain_id'])
    def push(self, params):
        """ Push notification

        Args:
            params (dict): {
                'protocol_id': 'str',
                'data': 'str',
                'message': 'dict',
                'notification_type', 'str',
                'domain_id': 'str'
            }

        Returns:
            None
        """
        domain_id = params['domain_id']
        protocol_id = params['protocol_id']
        notification_type = params.get('notification_type', 'INFO')
        data = params['data']
        message = params.get('message', {})

        protocol_mgr: ProtocolManager = self.locator.get_manager('ProtocolManager')
        protocol_vo = protocol_mgr.get_protocol(protocol_id, domain_id)
        secret_data = self.get_secret_data(protocol_vo, domain_id)

        self.push_queue(protocol_vo.protocol_id, data, secret_data, notification_type, message, domain_id)

    @transaction(append_meta={'authorization.scope': 'USER'})
    @check_required(['notifications', 'domain_id'])
    def delete(self, params):
        """ Delete notifications

        Args:
            params (dict): {
                'notifications': 'list',
                'domain_id': 'str'
            }

        Returns:
            None
        """
        filter_params = {
            'notification_id': params['notifications'],
            'domain_id': params['domain_id']
        }
        notification_vos = self.notification_mgr.filter_notifications(**filter_params)
        self.notification_mgr.delete_notification_by_vos(notification_vos)

    @transaction(append_meta={'authorization.scope': 'USER'})
    @check_required(['notifications', 'domain_id'])
    def set_read(self, params):
        """  Change the notifications to read status.

        Args:
            params (dict): {
                'notifications': 'list',
                'domain_id': 'str'
            }

        Returns:
            None
        """

        self.notification_mgr.set_read_notification(params['notifications'], params['domain_id'])

    @transaction(append_meta={'authorization.scope': 'USER'})
    @check_required(['notification_id', 'domain_id'])
    def get(self, params):
        """ Get Notification

        Args:
            params (dict): {
                'domain_id': 'str',
                'only': 'list',
                'set_read': 'bool'
            }

        Returns:
            notification_vo (object)
        """

        return self.notification_mgr.get_notification(params['notification_id'], params['domain_id'],
                                                      params.get('only'))

    @transaction(append_meta={'authorization.scope': 'USER'})
    @check_required(['domain_id'])
    @append_query_filter(['notification_id', 'topic', 'notification_type', 'notification_level', 'is_read',
                          'project_id', 'user_id', 'domain_id', 'user_self'])
    @append_keyword_filter(['notification_id', 'topic'])
    def list(self, params):
        """ List User Channels

        Args:
            params (dict): {
                'notification_id': 'str',
                'topic': 'str',
                'notification_type': 'str',
                'notification_level': 'str',
                'is_read': 'bool',
                'project_id': 'str',
                'user_id': 'str',
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.Query)'
            }

        Returns:
            results (list): 'list of user_channel_vo'
            total_count (int)
        """

        query = params.get('query', {})
        return self.notification_mgr.list_notifications(query)

    @transaction(append_meta={'authorization.scope': 'USER'})
    @check_required(['query', 'domain_id'])
    @append_query_filter(['domain_id', 'user_self'])
    @append_keyword_filter(['notification_id', 'topic'])
    def stat(self, params):
        """
        Args:
            params (dict): {
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)'
            }

        Returns:
            values (list): 'list of statistics data'
            total_count (int)
        """

        query = params.get('query', {})
        return self.notification_mgr.stat_notifications(query)

    def push_queue(self, protocol_id, channel_data, secret_data, notification_type, message, domain_id):
        task = {
            'name': 'dispatch_notification',
            'version': 'v1',
            'executionEngine': 'BaseWorker',
            'stages': [{
                'locator': 'SERVICE',
                'name': 'NotificationService',
                'metadata': self.transaction.meta,
                'method': 'dispatch_notification',
                'params': {
                    'protocol_id': protocol_id,
                    'channel_data': channel_data,
                    'secret_data': secret_data,
                    'notification_type': notification_type,
                    'message': message,
                    'domain_id': domain_id,
                }
            }]
        }

        _LOGGER.debug(f'[push_queue] task: {task}')
        queue.put('notification_q', utils.dump_json(task))

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    def delete_old_notifications(self, params):
        """ Delete old notifications

        Args:
            params (dict): {}

        Returns:
            None
        """
        now = datetime.datetime.now()
        now = now.replace(hour=0, minute=0, second=0, microsecond=0)
        condition_date = now - datetime.timedelta(days=OLD_NOTIFICATION_DAYS)
        condition_date_iso = utils.datetime_to_iso8601(condition_date)

        query = {'filter': [{'k': 'created_at', 'v': condition_date_iso, 'o': 'datetime_lte'}]}
        _LOGGER.debug(f"[delete_old_notifications] Query: {query}")

        notification_vos, total_count = self.notification_mgr.list_notifications(query)

        _LOGGER.debug(f"[delete_old_notifications] Old Notification Count: {total_count}")
        notification_vos.delete()

    def get_channel_data(self, channel_vo, protocol_vo, domain_id):
        secret_mgr: SecretManager = self.locator.get_manager('SecretManager')

        channel_data = None
        plugin_info = protocol_vo.plugin_info.to_dict()
        plugin_metadata = plugin_info.get('metadata', {})

        if plugin_metadata.get('data_type') == 'PLAIN_TEXT':
            channel_data = channel_vo.data
        elif plugin_metadata.get('data_type') == 'SECRET':
            channel_data = secret_mgr.get_secret_data(channel_vo.secret_id, domain_id)

        return channel_data

    def get_secret_data(self, protocol_vo, domain_id):
        secret_mgr: SecretManager = self.locator.get_manager('SecretManager')

        secret_data = {}
        plugin_info = protocol_vo.plugin_info.to_dict()

        if secret_id := plugin_info.get('secret_id'):
            secret_data = secret_mgr.get_secret_data(secret_id, domain_id)

        return secret_data

    def dispatch_notification(self, protocol_id, channel_data, secret_data, notification_type, message, domain_id):
        protocol_mgr: ProtocolManager = self.locator.get_manager('ProtocolManager')
        plugin_mgr: PluginManager = self.locator.get_manager('PluginManager')

        protocol_vo = protocol_mgr.get_protocol(protocol_id, domain_id)

        if protocol_vo.state == 'ENABLED':
            plugin_info = protocol_vo.plugin_info.to_dict()
            options = plugin_info.get('options', {})

            _LOGGER.debug(f'[Plugin Initialize] plugin_id: {plugin_info["plugin_id"]} | version: {plugin_info["version"]} '
                          f'| domain_id: {domain_id}')
            try:
                endpoint_info = plugin_mgr.initialize(plugin_info, domain_id)
                plugin_metadata = plugin_mgr.init_plugin(options)
                plugin_info['metadata'] = plugin_metadata

                if version := endpoint_info.get('updated_version'):
                    plugin_info['version'] = version

                protocol_mgr = self.locator.get_manager('ProtocolManager')
                protocol_mgr.update_protocol_by_vo({'plugin_info': plugin_info}, protocol_vo)

            except Exception as e:
                _LOGGER.error(f'[Notification] Plugin Error: {e}')

            self._dispatch_notification(protocol_vo, secret_data, channel_data, notification_type,
                                        message, options, plugin_mgr)

        else:
            _LOGGER.info('[Notification] Protocol is disabled. skip notification')

    def _dispatch_notification(self, protocol_vo, secret_data, channel_data, notification_type, message, options, plugin_mgr):
        month, date = self.get_month_date()
        noti_usage_vo, usage_month, usage_date = self.get_notification_usage(protocol_vo, month, date)

        try:
            self.check_quota(protocol_vo, usage_month, usage_date)
            plugin_mgr.dispatch_notification(secret_data, channel_data, notification_type, message, options)
            self.increment_usage(noti_usage_vo)
        except Exception as e:
            self.increment_fail_count(noti_usage_vo)

    def check_quota(self, protocol_vo, usage_month, usage_date, count=1):
        quota_mgr: QuotaManager = self.locator.get_manager('QuotaManager')

        query = {'filter': [{'k': 'protocol', 'v': protocol_vo, 'o': 'eq'}]}
        quota_results, quota_total_count = quota_mgr.list_quotas(query)

        if quota_total_count:
            limit = quota_results[0].limit
            self._check_quota_limit(protocol_vo.protocol_id, limit, usage_month, usage_date, count)
        else:
            # Check Default Quota
            plugin_id = protocol_vo.plugin_info.plugin_id

            if plugin_id in DEFAULT_QUOTA:
                limit = DEFAULT_QUOTA[plugin_id]
                self._check_quota_limit(protocol_vo.protocol_id, limit, usage_month, usage_date, count)

    def increment_usage(self, noti_usage_vo, count=1):
        noti_usage_mgr: NotificationUsageManager = self.locator.get_manager('NotificationUsageManager')
        _LOGGER.debug(f"[increment_usage] Incremental Usage Count - Protocol {noti_usage_vo.protocol_id} (count: {count})")
        noti_usage_mgr.incremental_notification_usage(noti_usage_vo, count)

    def increment_fail_count(self, noti_usage_vo, count=1):
        noti_usage_mgr: NotificationUsageManager = self.locator.get_manager('NotificationUsageManager')
        _LOGGER.debug(
            f"[increment_fail_count] Incremental Fail Count - Protocol {noti_usage_vo.protocol_id} (count: {count})")
        noti_usage_mgr.incremental_notification_fail_count(noti_usage_vo, count)

    def get_notification_usage(self, protocol_vo, month, date):
        usage_month = 0
        usage_date = 0
        noti_usage_vo = None

        noti_usage_mgr: NotificationUsageManager = self.locator.get_manager('NotificationUsageManager')

        month_query = {
            'filter': [
                {'k': 'protocol_id', 'v': protocol_vo.protocol_id, 'o': 'eq'},
                {'k': 'usage_month', 'v': month, 'o': 'eq'}
            ]
        }
        month_usage_results, month_usage_total_count = noti_usage_mgr.list_notification_usages(month_query)

        for _noti_usage in month_usage_results:
            usage_month += _noti_usage.count
            if _noti_usage.usage_date == date:
                usage_date = _noti_usage.count
                noti_usage_vo = _noti_usage

        if not noti_usage_vo:
            params = {
                'protocol_id': protocol_vo.protocol_id,
                'usage_month': month,
                'usage_date': date,
                'domain_id': protocol_vo.domain_id
            }
            noti_usage_vo = noti_usage_mgr.create_notification_usage(params)

        return noti_usage_vo, usage_month, usage_date

    @staticmethod
    def check_schedule_for_dispatch(is_scheduled, schedule):
        if is_scheduled:
            now_time = datetime.datetime.utcnow()

            valid_weekday = check_weekday_schedule(now_time, schedule.day_of_week)
            valid_time = check_time_schedule(now_time, schedule.start_hour, schedule.end_hour)

            _LOGGER.debug(f'Weekday: {valid_weekday} | Time {valid_time}')

            if valid_weekday and valid_time:
                return True
        else:
            return True

        return False

    @staticmethod
    def check_subscribe_for_dispatch(is_subscribe, subscriptions, topic):
        if is_subscribe is False:
            return True
        elif is_subscribe and topic in subscriptions:
            return True

        return False

    @staticmethod
    def check_notification_level_for_dispatch(notification_level, prj_channel_notification_level):
        if notification_level == 'ALL':
            return True
        elif prj_channel_notification_level == notification_level:
            return True

        return False

    @staticmethod
    def get_month_date():
        now = datetime.datetime.now()
        return now.strftime('%Y-%m'), now.strftime('%d')

    @staticmethod
    def _check_quota_limit(protocol_id, limit, usage_month, usage_date, count):
        if 'month' in limit:
            if limit['month'] != -1 and limit['month'] < (usage_month + count):
                raise ERROR_QUOTA_IS_EXCEEDED(protocol_id=protocol_id,
                                              limit=f'limit.month=({usage_month}/{limit["month"]})')

        if 'day' in limit:
            if limit['day'] != -1 and limit['day'] < (usage_date + count):
                raise ERROR_QUOTA_IS_EXCEEDED(protocol_id=protocol_id,
                                              limit=f'limit.day=({usage_date}/{limit["day"]})')

    @staticmethod
    def get_domain_name(domain_info):
        _tags = domain_info.get('tags', {})
        if description := _tags.get('description'):
            return description
        else:
            return domain_info['name']



from mongoengine import *
from spaceone.core.model.mongo_model import MongoModel
from spaceone.notification.model.protocol_model import Protocol


class Quota(MongoModel):
    quota_id = StringField(max_length=40, generate_id='quota', unique=True)
    protocol = ReferenceField('Protocol', reverse_delete_rule=DO_NOTHING)
    protocol_id = StringField(max_length=40)
    limit = DictField(default={})
    domain_id = StringField(max_length=40)

    meta = {
        'updatable_fields': [
            'limit'
        ],
        'minimal_fields': [
            'quota_id',
            'protocol_id',
            'domain_id',
        ],
        'ordering': ['quota_id'],
        'indexes': [
            # 'quota_id',
            'protocol_id',
            'domain_id'
        ]
    }



"""
Deprecated:
  Not used. Integrated with SpaceConnector.
"""

import logging

from google.protobuf.json_format import MessageToDict

from spaceone.core.connector import BaseConnector
from spaceone.core import pygrpc
from spaceone.core.utils import parse_endpoint
from spaceone.core.error import *

__all__ = ['RepositoryConnector']

_LOGGER = logging.getLogger(__name__)


class RepositoryConnector(BaseConnector):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._check_config()
        self._init_client()

    def _init_client(self):
        for version, uri in self.config['endpoint'].items():
            e = parse_endpoint(uri)
            self.client = pygrpc.client(endpoint=f'{e.get("hostname")}:{e.get("port")}', version=version)

    def _check_config(self):
        if 'endpoint' not in self.config:
            raise ERROR_CONNECTOR_CONFIGURATION(backend=self.__class__.__name__)

        if len(self.config['endpoint']) > 1:
            raise ERROR_CONNECTOR_CONFIGURATION(backend=self.__class__.__name__)

    def get_plugin(self, plugin_id, domain_id):
        response = self.client.Plugin.get({
            'plugin_id': plugin_id,
            'domain_id': domain_id
        }, metadata=self.transaction.get_connection_meta())

        return self._change_message(response)

    def get_plugin_versions(self, plugin_id, domain_id):
        response = self.client.Plugin.get_versions({
            'plugin_id': plugin_id,
            'domain_id': domain_id
        }, metadata=self.transaction.get_connection_meta())

        data = self._change_message(response)
        return data['results']

    def get_schema(self, schema_name, domain_id):
        response = self.client.Schema.get({
            'name': schema_name,
            'domain_id': domain_id
        }, metadata=self.transaction.get_connection_meta())

        return self._change_message(response)

    @staticmethod
    def _change_message(message):
        return MessageToDict(message, preserving_proto_field_name=True)



from spaceone.core import utils
from spaceone.core.service import *
from spaceone.notification.error import *
from spaceone.notification.lib.schedule import *
from spaceone.notification.lib.schema import *
from spaceone.notification.manager import IdentityManager
from spaceone.notification.manager import UserChannelManager
from spaceone.notification.model import UserChannel
from spaceone.notification.manager import ProtocolManager
from spaceone.notification.manager import SecretManager
from spaceone.notification.model.protocol_model import Protocol


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class UserChannelService(BaseService):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.user_channel_mgr: UserChannelManager = self.locator.get_manager('UserChannelManager')
        self.identity_mgr: IdentityManager = self.locator.get_manager('IdentityManager')
        self.protocol_mgr: ProtocolManager = self.locator.get_manager('ProtocolManager')
        self.secret_mgr: SecretManager = self.locator.get_manager('SecretManager')

    @transaction(append_meta={'authorization.scope': 'USER'})
    @check_required(['protocol_id', 'name', 'data', 'user_id', 'domain_id'])
    def create(self, params):
        """ Create user Channel

        Args:
            params (dict): {
                'protocol_id': 'str',
                'name': 'str',
                'data': 'dict',
                'is_subscribe': 'bool',
                'subscriptions': 'list',
                'is_scheduled': 'bool',
                'schedule': 'dict',
                'user_id': 'str',
                'tags': 'dict',
                'domain_id': 'str'
            }

        Returns:
            user_channel_vo (object)
        """

        protocol_id = params['protocol_id']
        domain_id = params['domain_id']
        data = params['data']
        user_id = params['user_id']
        is_subscribe = params.get('is_subscribe', False)
        is_scheduled = params.get('is_scheduled', False)

        if not is_subscribe:
            params['subscriptions'] = []

        if is_scheduled:
            validate_schedule(params.get('schedule', {}))
        else:
            params['schedule'] = None

        self.identity_mgr.get_resource(user_id, 'identity.User', domain_id)
        protocol_vo: Protocol = self.protocol_mgr.get_protocol(protocol_id, domain_id)

        if protocol_vo.state == 'DISABLED':
            raise ERROR_PROTOCOL_DISABLED()

        if protocol_vo.protocol_type == 'INTERNAL':
            raise ERROR_PROTOCOL_INTERNVAL()

        metadata = protocol_vo.plugin_info.metadata
        validate_json_schema(metadata.get('data', {}).get('schema', {}), data)

        if metadata['data_type'] == 'SECRET':
            new_secret_parameters = {
                'name': utils.generate_id('user-ch', 4),
                'secret_type': 'CREDENTIALS',
                'data': data,
                'domain_id': domain_id
            }

            user_channel_secret = self.secret_mgr.create_secret(new_secret_parameters)

            params.update({
                'secret_id': user_channel_secret['secret_id'],
                'data': {}
            })

        return self.user_channel_mgr.create_user_channel(params)

    @transaction(append_meta={'authorization.scope': 'USER'})
    @check_required(['user_channel_id', 'domain_id'])
    def update(self, params):
        """ Update user channel

        Args:
            params (dict): {
                'user_channel_id': 'str',
                'name': 'str',
                'data': 'dict',
                'tags': 'dict',
                'domain_id': 'str'
            }

        Returns:
            user_channel_vo (object)
        """

        user_channel_id = params['user_channel_id']
        domain_id = params['domain_id']

        user_channel_vo: UserChannel = self.user_channel_mgr.get_user_channel(user_channel_id, domain_id)

        if 'data' in params and user_channel_vo.secret_id:
            secret_params = {
                'secret_id': user_channel_vo.secret_id,
                'data': params['data'],
                'domain_id': domain_id
            }

            self.secret_mgr.update_secret_data(secret_params)
            params['data'] = {}

        return self.user_channel_mgr.update_user_channel_by_vo(params, user_channel_vo)

    @transaction(append_meta={'authorization.scope': 'USER'})
    @check_required(['user_channel_id', 'domain_id'])
    def set_schedule(self, params):
        """
            set_schedule
        Args:
            params (dict): {
                'user_channel_id': 'str',
                'is_scheduled': bool,
                'schedule': dict,
                'domain_id': 'str'
            }

        Returns:
            user_channel_vo (object)
        """

        user_channel_vo = self.user_channel_mgr.get_user_channel(params['user_channel_id'], params['domain_id'])

        is_scheduled = params.get('is_scheduled', False)

        if is_scheduled:
            validate_schedule(params.get('schedule', {}))
        else:
            params.update({
                'is_scheduled': False,
                'schedule': None
            })

        return self.user_channel_mgr.update_user_channel_by_vo(params, user_channel_vo)

    @transaction(append_meta={'authorization.scope': 'USER'})
    @check_required(['user_channel_id', 'domain_id'])
    def set_subscription(self, params):
        """
            set_subscription
        Args:
            params (dict): {
                'user_channel_id': 'str',
                'is_subscribe': bool,
                'subscriptions': list,
                'domain_id': 'str'
            }

        Returns:
            user_channel_vo (object)
        """
        if not params.get('is_subscribe', False):
            params.update({
                'is_subscribe': False,
                'subscriptions': []
            })

        return self.user_channel_mgr.update_user_channel(params)

    @transaction(append_meta={'authorization.scope': 'USER'})
    @check_required(['user_channel_id', 'domain_id'])
    def delete(self, params):
        """ Delete user channel

        Args:
            params (dict): {
                'user_channel_id': 'str',
                'domain_id': 'str'
            }

        Returns:
            None
        """
        user_channel_id = params['user_channel_id']
        domain_id = params['domain_id']

        user_channel_vo = self.user_channel_mgr.get_user_channel(user_channel_id, domain_id)

        if secret_id := user_channel_vo.secret_id:
            self.secret_mgr.delete_secret({'secret_id': secret_id, 'domain_id': domain_id})

        self.user_channel_mgr.delete_user_channel_by_vo(user_channel_vo)

    @transaction(append_meta={'authorization.scope': 'USER'})
    @check_required(['user_channel_id', 'domain_id'])
    def enable(self, params):
        """ Enable user channel

        Args:
            params (dict): {
                'user_channel_id': 'str',
                'domain_id': 'str'
            }

        Returns:
            user_channel_vo (object)
        """

        return self.user_channel_mgr.enable_user_channel(params['user_channel_id'], params['domain_id'])

    @transaction(append_meta={'authorization.scope': 'USER'})
    @check_required(['user_channel_id', 'domain_id'])
    def disable(self, params):
        """ Disable user channel

        Args:
            params (dict): {
                'user_channel_id': 'str',
                'domain_id': 'str'
            }

        Returns:
            user_channel_vo (object)
        """

        return self.user_channel_mgr.disable_user_channel(params['user_channel_id'], params['domain_id'])

    @transaction(append_meta={'authorization.scope': 'USER'})
    @check_required(['user_channel_id', 'domain_id'])
    def get(self, params):
        """ Get User Channel

        Args:
            params (dict): {
                'domain_id': 'str',
                'only': 'list'
            }

        Returns:
            user_channel_vo (object)
        """

        return self.user_channel_mgr.get_user_channel(params['user_channel_id'], params['domain_id'], params.get('only'))

    @transaction(append_meta={'authorization.scope': 'USER'})
    @check_required(['domain_id'])
    @append_query_filter(['user_channel_id', 'name', 'state', 'secret_id', 'protocol_id', 'user_id', 'domain_id',
                          'user_self'])
    @append_keyword_filter(['user_channel_id', 'name'])
    def list(self, params):
        """ List User Channels

        Args:
            params (dict): {
                'user_channel_id': 'str',
                'name': 'str',
                'state': 'str',
                'secret_id': 'str',
                'protocol_id': 'str',
                'user_id': 'str',
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.Query)'
            }

        Returns:
            results (list): 'list of user_channel_vo'
            total_count (int)
        """

        query = params.get('query', {})
        return self.user_channel_mgr.list_user_channels(query)

    @transaction(append_meta={'authorization.scope': 'USER'})
    @check_required(['query', 'domain_id'])
    @append_query_filter(['domain_id', 'user_self'])
    @append_keyword_filter(['user_channel_id', 'name'])
    def stat(self, params):
        """
        Args:
            params (dict): {
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)'
            }

        Returns:
            values (list): 'list of statistics data'
            total_count (int)
        """

        query = params.get('query', {})
        return self.user_channel_mgr.stat_user_channels(query)



from spaceone.core.error import *

class ERROR_PROTOCOL_DISABLED(ERROR_BASE):
    _message = 'Protocol was disabled.'

class ERROR_PROTOCOL_INTERNVAL(ERROR_BASE):
    _message = 'Protocol can be set when protocol type is EXTERNAL only'

class ERROR_NOT_SUPPORT_SCHEMA(ERROR_INVALID_ARGUMENT):
    _message = 'Protocol is not supported schema. (schema = {schema})'

class ERROR_WRONG_SCHEDULE_SETTINGS(ERROR_BASE):
    _message = "The schedule format is incorrect. (key = {key})"

class ERROR_WRONG_SCHEDULE_HOURS_SETTINGS(ERROR_BASE):
    _message = "The end hour must be greater than start hour. (start_hour={start_hour}, end_hour={end_hour})"

class ERROR_WRONG_SCHEDULE_DAY(ERROR_BASE):
    _message = "Invalid day to set schedule (day = {day})"

class ERROR_INVALID_DOMAIN(ERROR_BASE):
    _message = "Invalid resource_id (domain_id={resource_id})"



from spaceone.notification.connector.identity_connector import IdentityConnector
from spaceone.notification.connector.repository_connector import RepositoryConnector
from spaceone.notification.connector.secret_connector import SecretConnector
from spaceone.notification.connector.plugin_connector import PluginConnector
from spaceone.notification.connector.notification_plugin_connector import NotificationPluginConnector



from jsonschema import validate
from spaceone.notification.error import *

def validate_json_schema(data, schema):
    try:
        validate(instance=data, schema=schema)
    except Exception as e:
        if e.validator in ['pattern']:
            raise ERROR_JSON_SCHEMA_PATTERN(data=e.instance, schema_title=e.schema["title"])
        else:
            raise ERROR_JSON_SCHEMA(data=e.instance, message=e.message)



import logging
import copy
from spaceone.core import cache
from spaceone.core import utils
from spaceone.core import config
from spaceone.core.service import *
from spaceone.notification.error import *
from spaceone.notification.manager import RepositoryManager
from spaceone.notification.manager import ProtocolManager
from spaceone.notification.manager import PluginManager
from spaceone.notification.manager import ProjectChannelManager
from spaceone.notification.manager import UserChannelManager
from spaceone.notification.manager import SecretManager
from spaceone.notification.model import Protocol
from spaceone.notification.conf.protocol_conf import *


_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class ProtocolService(BaseService):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.protocol_mgr: ProtocolManager = self.locator.get_manager('ProtocolManager')

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['name', 'plugin_info', 'domain_id'])
    def create(self, params):
        """ Create Protocol
        Args:
            params (dict): {
                'name': 'str',
                'plugin_info': 'dict',
                'tags': 'dict',
                'domain_id': 'str',
                'upgrade_mode': 'str'
            }

        Returns:
            protocol_vo (object)
        """
        return self._create(params)

    def _create(self, params):
        domain_id = params['domain_id']
        plugin_info = params['plugin_info']

        self._check_plugin_info(plugin_info)
        _plugin = self._get_plugin(plugin_info, domain_id)
        plugin_capability = _plugin.get('capability', {})

        if 'supported_schema' in plugin_capability:
            params['capability'] = {'supported_schema': plugin_capability['supported_schema']}
        else:
            raise ERROR_WRONG_PLUGIN_SETTINGS(key='capability.supported_schema')

        _LOGGER.debug(f'[create] capability: {params["capability"]}')
        _LOGGER.debug(f'[create] name: {params["name"]}')

        plugin_metadata, endpoint_info = self._init_plugin(plugin_info, domain_id)

        request_plugin = {
            'plugin_id': plugin_info['plugin_id'],
            'options': plugin_info.get('options', {}),
            'metadata': plugin_metadata
        }

        if version := endpoint_info.get('updated_version', plugin_info.get('version')):
            request_plugin.update({
                'version': version
            })

        if 'secret_data' in plugin_info:
            secret_mgr: SecretManager = self.locator.get_manager('SecretManager')
            secret_params = {
                'name': utils.generate_id('secret-noti-proto', 4),
                'secret_type': 'CREDENTIALS',
                'data': plugin_info['secret_data'],
                'schema': plugin_info['schema'],
                'domain_id': domain_id
            }

            protocol_secret = secret_mgr.create_secret(secret_params)
            request_plugin.update({
                'secret_id': protocol_secret['secret_id'],
                'schema': plugin_info['schema']
            })

        params['plugin_info'] = request_plugin
        protocol_vo: Protocol = self.protocol_mgr.create_protocol(params)

        return protocol_vo

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['protocol_id', 'domain_id'])
    def update(self, params):
        """ Update protocol

        Args:
            params (dict): {
                'protocol_id': 'str',
                'name': 'str',
                'tags': 'dict',
                'domain_id': 'str'
            }

        Returns:
            protocol_vo (object)
        """

        domain_id = params['domain_id']
        protocol_id = params['protocol_id']

        protocol_vo = self.protocol_mgr.get_protocol(protocol_id, domain_id)

        if protocol_vo.protocol_type == 'INTERNAL':
            raise ERROR_NOT_ALLOWED_UPDATE_PROTOCOL_TYPE(protocol_id=protocol_id)

        return self.protocol_mgr.update_protocol_by_vo(params, protocol_vo)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['protocol_id', 'domain_id'])
    def update_plugin(self, params):
        """ Update protocol plugin

        Args:
            params (dict): {
                'protocol_id': 'str',
                'version': 'str',
                'options': 'dict',
                'domain_id': 'str'
            }

        Returns:
            protocol_vo (object)
        """
        protocol_id = params['protocol_id']
        domain_id = params['domain_id']

        protocol_vo = self.protocol_mgr.get_protocol(protocol_id, domain_id)

        plugin_info = protocol_vo.plugin_info.to_dict()

        if plugin_info['upgrade_mode'] == 'AUTO':
            plugin_metadata, endpoint_info = self._init_plugin(plugin_info, domain_id)

            plugin_info['metadata'] = plugin_metadata

            if version := endpoint_info.get('updated_version'):
                plugin_info['version'] = version
        else:
            if version := params.get('version'):
                # Update plugin_version
                plugin_id = plugin_info['plugin_id']

                repo_mgr = self.locator.get_manager('RepositoryManager')
                repo_mgr.check_plugin_version(plugin_id, version, domain_id)

                plugin_info['version'] = version
                plugin_info['metadata'] = self._init_plugin(plugin_info, domain_id)

        if options := params.get('options', {}):
            # Overwrite
            plugin_info['options'] = options

        params = {
            'protocol_id': protocol_id,
            'domain_id': domain_id,
            'plugin_info': plugin_info
        }

        _LOGGER.debug(f'[update_plugin] {plugin_info}')
        return self.protocol_mgr.update_protocol_by_vo(params, protocol_vo)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['protocol_id', 'domain_id'])
    def delete(self, params):
        """ Delete protocol

        Args:
            params (dict): {
                'protocol_id': 'str',
                'domain_id': 'str'
            }

        Returns:
            None
        """
        protocol_id = params['protocol_id']
        domain_id = params['domain_id']

        protocol_vo: Protocol = self.protocol_mgr.get_protocol(protocol_id, domain_id)
        self.check_existed_channel_using_protocol(protocol_vo)

        if secret_id := protocol_vo.plugin_info.secret_id:
            secret_mgr: SecretManager = self.locator.get_manager('SecretManager')
            secret_mgr.delete_secret({'secret_id': secret_id, 'domain_id': domain_id})

        return self.protocol_mgr.delete_protocol_by_vo(protocol_vo)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['protocol_id', 'domain_id'])
    def enable(self, params):
        """ Enable protocol

        Args:
            params (dict): {
                'protocol_id': 'str',
                'domain_id': 'str'
            }

        Returns:
            protocol_vo (object)
        """

        return self.protocol_mgr.enable_protocol(params['protocol_id'], params['domain_id'])

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['protocol_id', 'domain_id'])
    def disable(self, params):
        """ Disable protocol

        Args:
            params (dict): {
                'protocol_id': 'str',
                'domain_id': 'str'
            }

        Returns:
            protocol_vo (object)
        """

        return self.protocol_mgr.disable_protocol(params['protocol_id'], params['domain_id'])

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['protocol_id', 'domain_id'])
    def get(self, params):
        """ Get Protocol

        Args:
            params (dict): {
                'domain_id': 'str',
                'only': 'list'
            }

        Returns:
            domain_vo (object)
        """
        protocol_id = params['protocol_id']
        domain_id = params['domain_id']

        # Create Default Protocol if protocol is not exited
        self._create_default_protocol(domain_id)
        self._initialize_protocols(domain_id)

        return self.protocol_mgr.get_protocol(protocol_id, domain_id, params.get('only'))

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['domain_id'])
    @append_query_filter(['protocol_id', 'name', 'state', 'protocol_type', 'domain_id'])
    @append_keyword_filter(['protocol_id', 'name'])
    def list(self, params):
        """ List protocol

        Args:
            params (dict): {
                'protocol_id': 'str',
                'name': 'str',
                'state': 'str',
                'protocol_type',
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.Query)'
            }

        Returns:
            results (list): 'list of protocol_vo'
            total_count (int)
        """
        domain_id = params['domain_id']
        query = params.get('query', {})

        # Create Default Protocol if protocol is not exited
        self._create_default_protocol(domain_id)
        self._initialize_protocols(domain_id)

        return self.protocol_mgr.list_protocols(query)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['query', 'domain_id'])
    @append_query_filter(['domain_id'])
    @append_keyword_filter(['protocol_id', 'name'])
    def stat(self, params):
        """
        Args:
            params (dict): {
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)'
            }

        Returns:
            values (list): 'list of statistics data'
            total_count (int)
        """

        query = params.get('query', {})
        return self.protocol_mgr.stat_protocols(query)

    def _get_plugin(self, plugin_info, domain_id):
        plugin_id = plugin_info['plugin_id']

        repo_mgr: RepositoryManager = self.locator.get_manager('RepositoryManager')
        plugin_info = repo_mgr.get_plugin(plugin_id, domain_id)

        if version := plugin_info.get('version'):
            repo_mgr.check_plugin_version(plugin_id, version, domain_id)

        return plugin_info

    def _init_plugin(self, plugin_info, domain_id):
        options = plugin_info.get('options', {})

        plugin_mgr: PluginManager = self.locator.get_manager('PluginManager')
        endpoint_info = plugin_mgr.initialize(plugin_info, domain_id)
        metadata = plugin_mgr.init_plugin(options)

        return metadata, endpoint_info

    def check_existed_channel_using_protocol(self, protocol_vo):
        project_channel_mgr: ProjectChannelManager = self.locator.get_manager('ProjectChannelManager')
        user_channel_mgr: UserChannelManager = self.locator.get_manager('UserChannelManager')

        query = {'filter': [{'k': 'protocol_id', 'v': protocol_vo.protocol_id, 'o': 'eq'}]}

        project_channel_vos, prj_ch_total_count = project_channel_mgr.list_project_channels(query)
        user_channel_vos, user_ch_total_count = user_channel_mgr.list_user_channels(query)

        if prj_ch_total_count > 0 or user_ch_total_count > 0:
            raise EROR_DELETE_PROJECT_EXITED_CHANNEL(protocol_id=protocol_vo.protocol_id)

    @cache.cacheable(key='default-protocol:{domain_id}', expire=300)
    def _create_default_protocol(self, domain_id):
        _LOGGER.debug(f'[_create_default_protocol] domain_id: {domain_id}')

        query = {'filter': [{'k': 'domain_id', 'v': domain_id, 'o': 'eq'}]}
        protocol_vos, total_count = self.protocol_mgr.list_protocols(query)

        installed_protocol_names = [protocol_vo.name for protocol_vo in protocol_vos]
        _LOGGER.debug(f'[_create_default_protocol] Installed Plugins : {installed_protocol_names}')

        for _protocol_data in DEFAULT_INTERNAL_PROTOCOLS:
            if _protocol_data['name'] not in installed_protocol_names:
                _LOGGER.debug(f'Create default protocol: {_protocol_data["name"]}')
                _protocol_data['domain_id'] = domain_id
                self.protocol_mgr.create_protocol(_protocol_data)

        return True

    @cache.cacheable(key='init-protocol:{domain_id}', expire=300)
    def _initialize_protocols(self, domain_id):
        _LOGGER.debug(f'[_initialize_protocol] domain_id: {domain_id}')

        query = {'filter': [{'k': 'domain_id', 'v': domain_id, 'o': 'eq'}]}
        protocol_vos, total_count = self.protocol_mgr.list_protocols(query)

        installed_protocol_ids = [protocol_vo.plugin_info.plugin_id for protocol_vo in protocol_vos]
        _LOGGER.debug(f'[_initialize_protocol] Installed Plugins : {installed_protocol_ids}')

        global_conf = config.get_global()
        for _protocol_data in global_conf.get('INSTALLED_PROTOCOL_PLUGINS', []):
            if _protocol_data['plugin_info']['plugin_id'] not in installed_protocol_ids:
                try:
                    _LOGGER.debug(f'[_initialize_protocol] Create init protocol: {_protocol_data["plugin_info"]["plugin_id"]}')
                    _protocol_data['domain_id'] = domain_id
                    self._create(_protocol_data)
                except Exception as e:
                    _LOGGER.error(f'[_initialize_protocol] {e}')

        return True

    @staticmethod
    def _check_plugin_info(plugin_info_params):
        if 'plugin_id' not in plugin_info_params:
            raise ERROR_REQUIRED_PARAMETER(key='plugin_info.plugin_id')

        if 'secret_data' in plugin_info_params and 'schema' not in plugin_info_params:
            raise ERROR_REQUIRED_PARAMETER(key='plugin_info.schema')

        if 'upgrade_mode' in plugin_info_params and plugin_info_params['upgrade_mode'] == 'MANUAL':
            if 'version' not in plugin_info_params:
                raise ERROR_REQUIRED_PARAMETER(key='plugin_info.version')



from spaceone.notification.service.notification_service import NotificationService
from spaceone.notification.service.protocol_service import ProtocolService
from spaceone.notification.service.project_channel_service import ProjectChannelService
from spaceone.notification.service.user_channel_service import UserChannelService
from spaceone.notification.service.quota_service import QuotaService
from spaceone.notification.service.notification_usage_service import NotificationUsageService



import logging

from spaceone.core.manager import BaseManager
from spaceone.notification.model.user_channel_model import UserChannel

_LOGGER = logging.getLogger(__name__)


class UserChannelManager(BaseManager):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.user_channel_model: UserChannel = self.locator.get_model('UserChannel')

    def create_user_channel(self, params):
        def _rollback(vo):
            _LOGGER.info(f'[create_user_channel._rollback] Delete User Channel : {vo.name} ({vo.user_channel_id})')
            vo.delete()

        user_channel_vo: UserChannel = self.user_channel_model.create(params)

        self.transaction.add_rollback(_rollback, user_channel_vo)

        return user_channel_vo

    def update_user_channel(self, params):
        user_channel_vo = self.get_user_channel(params['user_channel_id'], params['domain_id'])
        return self.update_user_channel_by_vo(params, user_channel_vo)

    def update_user_channel_by_vo(self, params, user_channel_vo):
        def _rollback(old_data):
            _LOGGER.info(f'[user_user_channel._rollback] '
                         f'Revert Data : {old_data["name"]} '
                         f'({old_data["user_channel_id"]})')
            user_channel_vo.update(old_data)

        self.transaction.add_rollback(_rollback, user_channel_vo.to_dict())
        return user_channel_vo.update(params)

    def delete_user_channel(self, user_channel_id, domain_id):
        user_channel_vo: UserChannel = self.get_user_channel(user_channel_id, domain_id)
        self.delete_user_channel_by_vo(user_channel_vo)

    def enable_user_channel(self, user_channel_id, domain_id):
        def _rollback(old_data):
            _LOGGER.info(f'[enable_user_channel._rollback] '
                         f'Revert Data : {old_data["name"]} ({old_data["user_channel_id"]})')
            user_channel_vo.update(old_data)

        user_channel_vo: UserChannel = self.get_user_channel(user_channel_id, domain_id)

        if user_channel_vo.state != 'ENABLED':
            self.transaction.add_rollback(_rollback, user_channel_vo.to_dict())
            user_channel_vo.update({'state': 'ENABLED'})

        return user_channel_vo

    def disable_user_channel(self, user_channel_id, domain_id):
        def _rollback(old_data):
            _LOGGER.info(f'[disable_user_channel._rollback] '
                         f'Revert Data : {old_data["name"]} ({old_data["user_channel_id"]})')
            user_channel_vo.update(old_data)

        user_channel_vo: UserChannel = self.get_user_channel(user_channel_id, domain_id)

        if user_channel_vo.state != 'DISABLED':
            self.transaction.add_rollback(_rollback, user_channel_vo.to_dict())
            user_channel_vo.update({'state': 'DISABLED'})

        return user_channel_vo

    def get_user_channel(self, user_channel_id, domain_id, only=None):
        return self.user_channel_model.get(user_channel_id=user_channel_id, domain_id=domain_id, only=only)

    def list_user_channels(self, query):
        return self.user_channel_model.query(**query)

    def stat_user_channels(self, query):
        return self.user_channel_model.stat(**query)

    @staticmethod
    def delete_user_channel_by_vo(user_channel_vo):
        user_channel_vo.delete()


from spaceone.api.notification.v1 import project_channel_pb2, project_channel_pb2_grpc
from spaceone.core.pygrpc import BaseAPI


class ProjectChannel(BaseAPI, project_channel_pb2_grpc.ProjectChannelServicer):

    pb2 = project_channel_pb2
    pb2_grpc = project_channel_pb2_grpc

    def create(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProjectChannelService', metadata) as project_channel_svc:
            return self.locator.get_info('ProjectChannelInfo', project_channel_svc.create(params))

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProjectChannelService', metadata) as project_channel_svc:
            return self.locator.get_info('ProjectChannelInfo', project_channel_svc.update(params))

    def set_schedule(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProjectChannelService', metadata) as project_channel_svc:
            return self.locator.get_info('ProjectChannelInfo', project_channel_svc.set_schedule(params))

    def set_subscription(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProjectChannelService', metadata) as project_channel_svc:
            return self.locator.get_info('ProjectChannelInfo', project_channel_svc.set_subscription(params))

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProjectChannelService', metadata) as project_channel_svc:
            project_channel_svc.delete(params)
            return self.locator.get_info('EmptyInfo')

    def enable(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProjectChannelService', metadata) as project_channel_svc:
            return self.locator.get_info('ProjectChannelInfo', project_channel_svc.enable(params))

    def disable(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProjectChannelService', metadata) as project_channel_svc:
            return self.locator.get_info('ProjectChannelInfo', project_channel_svc.disable(params))

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProjectChannelService', metadata) as project_channel_svc:
            return self.locator.get_info('ProjectChannelInfo', project_channel_svc.get(params))

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProjectChannelService', metadata) as project_channel_svc:
            project_channel_vos, total_count = project_channel_svc.list(params)
            return self.locator.get_info('ProjectChannelsInfo',
                                         project_channel_vos,
                                         total_count,
                                         minimal=self.get_minimal(params))

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProjectChannelService', metadata) as project_channel_svc:
            return self.locator.get_info('StatisticsInfo', project_channel_svc.stat(params))



import logging

from spaceone.core.manager import BaseManager
from spaceone.core.connector.space_connector import SpaceConnector
from spaceone.notification.error import *


_LOGGER = logging.getLogger(__name__)


class SecretManager(BaseManager):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.secret_connector: SpaceConnector = self.locator.get_connector('SpaceConnector', service='secret')

    def create_secret(self, params):
        return self.secret_connector.dispatch('Secret.create', params)

    def update_secret(self, params):
        return self.secret_connector.dispatch('Secret.update', params)

    def update_secret_data(self, params):
        return self.secret_connector.dispatch('Secret.update_data', params)

    def delete_secret(self, params):
        return self.secret_connector.dispatch('Secret.delete', params)

    def list_secrets(self, query, domain_id):
        return self.secret_connector.dispatch('Secret.list', {'query': query, 'domain_id': domain_id})

    def get_secret_data(self, secret_id, domain_id):
        response = self.secret_connector.dispatch('Secret.get_data', {'secret_id': secret_id, 'domain_id': domain_id})
        return response['data']

    def get_plugin_secret_data(self, secret_id, supported_schema, domain_id):
        secret_query = self._make_query(supported_schema=supported_schema, secret_id=secret_id)
        response = self.list_secrets(secret_query, domain_id)

        if response.get('total_count', 0) == 0:
            raise ERROR_NOT_FOUND(key='plugin_info.secret_id', value=secret_id)

        return self.get_secret_data(secret_id, domain_id)

    @staticmethod
    def _make_query(**secret_filter):
        supported_schema = secret_filter.get('supported_schema')
        secret_id = secret_filter.get('secret_id')
        service_account_id = secret_filter.get('service_account_id')
        project_id = secret_filter.get('project_id')
        provider = secret_filter.get('provider')
        secrets = secret_filter.get('secrets')

        query = {
            'filter': []
        }

        if supported_schema:
            query['filter'].append({
                'k': 'schema',
                'v': supported_schema,
                'o': 'in'
            })

        if secret_id:
            query['filter'].append({
                'k': 'secret_id',
                'v': secret_id,
                'o': 'eq'
            })

        if provider:
            query['filter'].append({
                'k': 'provider',
                'v': provider,
                'o': 'eq'
            })

        if service_account_id:
            query['filter'].append({
                'k': 'service_account_id',
                'v': service_account_id,
                'o': 'eq'
            })

        if project_id:
            query['filter'].append({
                'k': 'project_id',
                'v': project_id,
                'o': 'eq'
            })

        if secrets:
            query['filter'].append({
                'k': 'secret_id',
                'v': secrets,
                'o': 'in'
            })

        return query



from spaceone.api.notification.v1 import quota_pb2, quota_pb2_grpc
from spaceone.core.pygrpc import BaseAPI


class Quota(BaseAPI, quota_pb2_grpc.QuotaServicer):

    pb2 = quota_pb2
    pb2_grpc = quota_pb2_grpc

    def create(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('QuotaService', metadata) as quota_svc:
            return self.locator.get_info('QuotaInfo', quota_svc.create(params))

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('QuotaService', metadata) as quota_svc:
            return self.locator.get_info('QuotaInfo', quota_svc.update(params))

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('QuotaService', metadata) as quota_svc:
            quota_svc.delete(params)
            return self.locator.get_info('EmptyInfo')

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('QuotaService', metadata) as quota_svc:
            return self.locator.get_info('QuotaInfo', quota_svc.get(params))

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('QuotaService', metadata) as quota_svc:
            quota_vos, total_count = quota_svc.list(params)
            return self.locator.get_info('QuotasInfo',
                                         quota_vos,
                                         total_count,
                                         minimal=self.get_minimal(params))

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('QuotaService', metadata) as quota_svc:
            return self.locator.get_info('StatisticsInfo', quota_svc.stat(params))



import logging
from spaceone.core.manager import BaseManager
from spaceone.notification.model.notification_usage_model import NotificationUsage

_LOGGER = logging.getLogger(__name__)


class NotificationUsageManager(BaseManager):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.noti_usage_model: NotificationUsage = self.locator.get_model('NotificationUsage')

    def create_notification_usage(self, params):
        def _rollback(noti_usage_vo):
            _LOGGER.info(f'[create_notification_usage._rollback]'
                         f'Delete Notification Usage : {noti_usage_vo.protocol_id}'
                         f'({noti_usage_vo.protocol_id})')
            noti_usage_vo.delete()

        noti_usage_vo: NotificationUsage = self.noti_usage_model.create(params)
        self.transaction.add_rollback(_rollback, noti_usage_vo)

        return noti_usage_vo

    def get_notification_usage(self, protocol_id, usage_month, usage_date, domain_id, only=None):
        return self.noti_usage_model.get(protocol_id=protocol_id,
                                         usage_month=usage_month, usage_date=usage_date,
                                         domain_id=domain_id, only=only)

    def list_notification_usages(self, query={}):
        return self.noti_usage_model.query(**query)

    def stat_notification_usages(self, query):
        return self.noti_usage_model.stat(**query)

    @staticmethod
    def incremental_notification_usage(noti_usage_vo, count):
        noti_usage_vo.increment("count", count)

    @staticmethod
    def incremental_notification_fail_count(noti_usage_vo, count):
        noti_usage_vo.increment("fail_count", count)



import functools

from spaceone.api.notification.v1 import quota_pb2
from spaceone.core.pygrpc.message_type import *
from spaceone.notification.model.quota_model import Quota

__all__ = ['QuotaInfo', 'QuotasInfo']


def QuotaInfo(quota_vo: Quota, minimal=False):
    info = {
        'quota_id': quota_vo.quota_id,
        'protocol_id': quota_vo.protocol_id,
        'domain_id': quota_vo.domain_id
    }

    if not minimal:
        info.update({'limit': change_struct_type(quota_vo.limit)})

    return quota_pb2.QuotaInfo(**info)


def QuotasInfo(quota_vos, total_count, **kwargs):
    results = list(map(functools.partial(QuotaInfo, **kwargs), quota_vos))
    return quota_pb2.QuotasInfo(results=results, total_count=total_count)



from mongoengine import *
from spaceone.core.model.mongo_model import MongoModel


class Notification(MongoModel):
    notification_id = StringField(max_length=40, generate_id='notification', unique=True)
    topic = StringField(max_length=255)
    message = DictField()
    notification_type = StringField(max_length=20, default='INFO')
    notification_level = StringField(max_length=40, default='ALL')
    is_read = BooleanField(default=False)
    user_id = StringField(max_length=60)
    domain_id = StringField(max_length=40)
    created_at = DateTimeField(auto_now_add=True)

    meta = {
        'updatable_fields': [
            'is_read'
        ],
        'minimal_fields': [
            'notification_id',
            'topic',
            'notification_type',
            'notification_level',
        ],
        'indexes': [
            # 'notification_id',
            'topic',
            'notification_type',
            'notification_level',
            'is_read',
            'user_id',
            'domain_id'
        ]
    }



from spaceone.notification.interface.grpc.v1.protocol import Protocol
from spaceone.notification.interface.grpc.v1.user_channel import UserChannel
from spaceone.notification.interface.grpc.v1.project_channel import ProjectChannel
from spaceone.notification.interface.grpc.v1.notification import Notification
from spaceone.notification.interface.grpc.v1.notification_usage import NotificationUsage
from spaceone.notification.interface.grpc.v1.quota import Quota



from spaceone.api.notification.v1 import notification_usage_pb2, notification_usage_pb2_grpc
from spaceone.core.pygrpc import BaseAPI


class NotificationUsage(BaseAPI, notification_usage_pb2_grpc.NotificationUsageServicer):

    pb2 = notification_usage_pb2
    pb2_grpc = notification_usage_pb2_grpc

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('NotificationUsageService', metadata) as notification_usage_svc:
            notification_usage_vos, total_count = notification_usage_svc.list(params)
            return self.locator.get_info('NotificationUsagesInfo',
                                         notification_usage_vos,
                                         total_count,
                                         minimal=self.get_minimal(params))

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('NotificationUsageService', metadata) as notification_usage_svc:
            return self.locator.get_info('StatisticsInfo', notification_usage_svc.stat(params))



import logging

from spaceone.core.manager import BaseManager
from spaceone.core.connector.space_connector import SpaceConnector
from spaceone.notification.connector.notification_plugin_connector import NotificationPluginConnector

_LOGGER = logging.getLogger(__name__)


class PluginManager(BaseManager):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.plugin_connector: SpaceConnector = self.locator.get_connector('SpaceConnector', service='plugin')
        self.noti_plugin_connector: NotificationPluginConnector = self.locator.get_connector(
            'NotificationPluginConnector')

    def initialize(self, plugin_info, domain_id):
        _LOGGER.debug(f'[initialize] plugin_info: {plugin_info}')

        plugin_id = plugin_info['plugin_id']
        upgrade_mode = plugin_info.get('upgrade_mode', 'AUTO')

        if upgrade_mode == 'AUTO':
            endpoint_response = self.plugin_connector.dispatch('Plugin.get_plugin_endpoint',
                                                               {'plugin_id': plugin_id,
                                                                'domain_id': domain_id,
                                                                'upgrade_mode': 'AUTO'})
        else:
            endpoint_response = self.plugin_connector.dispatch('Plugin.get_plugin_endpoint',
                                                               {'plugin_id': plugin_id,
                                                                'domain_id': domain_id,
                                                                'version': plugin_info.get('version')})

        endpoint = endpoint_response['endpoint']
        _LOGGER.debug(f'[init_plugin] endpoint: {endpoint}')
        self.noti_plugin_connector.initialize(endpoint)

        return endpoint_response

    def init_plugin(self, options):
        plugin_info = self.noti_plugin_connector.init(options)

        _LOGGER.debug(f'[plugin_info] {plugin_info}')
        return plugin_info.get('metadata', {})

    def verify_plugin(self, options, secret_data):
        self.noti_plugin_connector.verify(options, secret_data)

    def dispatch_notification(self, secret_data, channel_data, notification_type, message, options):
        return self.noti_plugin_connector.dispatch_notification(secret_data, channel_data, notification_type,
                                                                message, options)



from spaceone.notification.error import *

def validate_schedule(schedule):
    if 'day_of_week' not in schedule:
        raise ERROR_REQUIRED_PARAMETER(key='schedule.day_of_week')

    schedule['start_hour'] = schedule.get('start_hour', 0)
    schedule['end_hour'] = schedule.get('end_hour', 0)

    if schedule['start_hour'] < 0 or schedule['start_hour'] > 24:
        raise ERROR_WRONG_SCHEDULE_SETTINGS(key='schedule.start_hour')

    if schedule['end_hour'] < 1 or schedule['end_hour'] > 25:
        raise ERROR_WRONG_SCHEDULE_SETTINGS(key='schedule.end_hour')

    if schedule['start_hour'] == schedule['end_hour']:
        raise ERROR_WRONG_SCHEDULE_SETTINGS(key='schedule.start_hour')

def check_weekday_schedule(now_time, day_of_week):
    DAYS = ['MON', 'TUE', 'WED', 'THU', 'FRI', 'SAT', 'SUN']

    week_day = now_time.weekday()

    if DAYS[week_day] in day_of_week:
        return True

    return False

def check_time_schedule(now_time, start_hour, end_hour):
    hour = now_time.hour

    if start_hour < end_hour:
        if start_hour <= hour < end_hour:
            return True
        else:
            return False
    else:
        if start_hour <= hour or hour < end_hour:
            return True
        else:
            return False



from spaceone.api.notification.v1 import notification_pb2, notification_pb2_grpc
from spaceone.core.pygrpc import BaseAPI


class Notification(BaseAPI, notification_pb2_grpc.NotificationServicer):

    pb2 = notification_pb2
    pb2_grpc = notification_pb2_grpc

    def create(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('NotificationService', metadata) as notification_svc:
            notification_svc.create(params)
            return self.locator.get_info('EmptyInfo')

    def push(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('NotificationService', metadata) as notification_svc:
            notification_svc.push(params)
            return self.locator.get_info('EmptyInfo')

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('NotificationService', metadata) as notification_svc:
            notification_svc.delete(params)
            return self.locator.get_info('EmptyInfo')

    def delete_all(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('NotificationService', metadata) as notification_svc:
            notification_svc.delete_all(params)
            return self.locator.get_info('EmptyInfo')

    def set_read(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('NotificationService', metadata) as notification_svc:
            notification_svc.set_read(params)
            return self.locator.get_info('EmptyInfo')

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('NotificationService', metadata) as notification_svc:
            return self.locator.get_info('NotificationInfo', notification_svc.get(params))

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('NotificationService', metadata) as notification_svc:
            notification_vos, total_count = notification_svc.list(params)
            return self.locator.get_info('NotificationsInfo',
                                         notification_vos,
                                         total_count,
                                         minimal=self.get_minimal(params))

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('NotificationService', metadata) as notification_svc:
            return self.locator.get_info('StatisticsInfo', notification_svc.stat(params))



import logging
from spaceone.core.service import *
from spaceone.notification.manager import NotificationUsageManager

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class NotificationUsageService(BaseService):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.noti_usage_mgr: NotificationUsageManager = self.locator.get_manager('NotificationUsageManager')

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['domain_id'])
    @append_query_filter(['protocol_id', 'domain_id'])
    def list(self, params):
        """ List Notification Usages

        Args:
            params (dict): {
                'protocol_id': 'str',
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.Query)',
            }

        Returns:
            results (list): 'list of user_channel_vo'
            total_count (int)
        """

        query = params.get('query', {})
        return self.noti_usage_mgr.list_notification_usages(query)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['query', 'domain_id'])
    @append_query_filter(['domain_id'])
    def stat(self, params):
        """
        Args:
            params (dict): {
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)',
                'user_self': 'str', // from meta
            }

        Returns:
            values (list): 'list of statistics data'
            total_count (int)
        """

        query = params.get('query', {})
        return self.noti_usage_mgr.stat_notification_usages(query)



import logging

from spaceone.core.manager import BaseManager
from spaceone.notification.model.project_channel_model import ProjectChannel

_LOGGER = logging.getLogger(__name__)


class ProjectChannelManager(BaseManager):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.project_channel_model: ProjectChannel = self.locator.get_model('ProjectChannel')

    def create_project_channel(self, params):
        def _rollback(vo):
            _LOGGER.info(f'[create_project_channel._rollback] Delete Project Channel : {vo.name} ({vo.project_channel_id})')
            vo.delete()

        project_channel_vo: ProjectChannel = self.project_channel_model.create(params)

        self.transaction.add_rollback(_rollback, project_channel_vo)

        return project_channel_vo

    def update_project_channel(self, params):
        project_channel_vo = self.get_project_channel(params['project_channel_id'], params['domain_id'])
        return self.update_project_channel_by_vo(params, project_channel_vo)

    def update_project_channel_by_vo(self, params, project_channel_vo):
        def _rollback(old_data):
            _LOGGER.info(f'[update_project_channel._rollback] Revert Data : {old_data["name"]} '
                         f'({old_data["project_channel_id"]})')
            project_channel_vo.update(old_data)

        self.transaction.add_rollback(_rollback, project_channel_vo.to_dict())
        return project_channel_vo.update(params)

    def delete_project_channel(self, project_channel_id, domain_id):
        project_channel_vo: ProjectChannel = self.get_project_channel(project_channel_id, domain_id)
        self.delete_project_channel_by_vo(project_channel_vo)

    def enable_project_channel(self, project_channel_id, domain_id):
        def _rollback(old_data):
            _LOGGER.info(f'[enable_project_channel._rollback] '
                         f'Revert Data : {old_data["name"]} ({old_data["project_channel_id"]})')
            project_channel_vo.update(old_data)

        project_channel_vo: ProjectChannel = self.get_project_channel(project_channel_id, domain_id)

        if project_channel_vo.state != 'ENABLED':
            self.transaction.add_rollback(_rollback, project_channel_vo.to_dict())
            project_channel_vo.update({'state': 'ENABLED'})

        return project_channel_vo

    def disable_project_channel(self, project_channel_id, domain_id):
        def _rollback(old_data):
            _LOGGER.info(f'[disable_project_channel._rollback] '
                         f'Revert Data : {old_data["name"]} ({old_data["project_channel_id"]})')
            project_channel_vo.update(old_data)

        project_channel_vo: ProjectChannel = self.get_project_channel(project_channel_id, domain_id)

        if project_channel_vo.state != 'DISABLED':
            self.transaction.add_rollback(_rollback, project_channel_vo.to_dict())
            project_channel_vo.update({'state': 'DISABLED'})

        return project_channel_vo

    def get_project_channel(self, project_channel_id, domain_id, only=None):
        return self.project_channel_model.get(project_channel_id=project_channel_id, domain_id=domain_id, only=only)

    def list_project_channels(self, query):
        return self.project_channel_model.query(**query)

    def stat_project_channels(self, query):
        return self.project_channel_model.stat(**query)

    @staticmethod
    def delete_project_channel_by_vo(project_channel_vo):
        project_channel_vo.delete()